# Copyright Contributors to the DNF5 project.
# D-Bus sphinx domain extension
#
# Copyright (C) 2021, Red Hat Inc.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Author: Marc-André Lureau <marcandre.lureau@redhat.com>

from typing import (
    Any,
    Dict,
    Iterable,
    Iterator,
    List,
    NamedTuple,
    Optional,
    Tuple,
    cast,
)

from docutils import nodes
from docutils.nodes import Element, Node
from docutils.parsers.rst import directives
from sphinx import addnodes
from sphinx.addnodes import desc_signature, pending_xref
from sphinx.directives import ObjectDescription
from sphinx.domains import Domain, Index, IndexEntry, ObjType
from sphinx.locale import _
from sphinx.roles import XRefRole
from sphinx.util import nodes as node_utils
from sphinx.util.docfields import Field, TypedField
from sphinx.util.typing import OptionSpec


class DBusDescription(ObjectDescription[str]):
    """Base class for DBus objects"""

    option_spec: OptionSpec = ObjectDescription.option_spec.copy()
    option_spec.update(
        {
            "deprecated": directives.flag,
        }
    )

    def get_index_text(self, modname: str, name: str) -> str:
        """Return the text for the index entry of the object."""
        raise NotImplementedError("must be implemented in subclasses")

    def add_target_and_index(
        self, name: str, sig: str, signode: desc_signature
    ) -> None:
        ifacename = self.env.ref_context.get("dbus:interface")
        node_id = name
        if ifacename:
            node_id = f"{ifacename}.{node_id}"

        signode["names"].append(name)
        signode["ids"].append(node_id)

        if "noindexentry" not in self.options:
            indextext = self.get_index_text(ifacename, name)
            if indextext:
                self.indexnode["entries"].append(
                    ("single", indextext, node_id, "", None)
                )

        domain = cast(DBusDomain, self.env.get_domain("dbus"))
        domain.note_object(name, self.objtype, node_id, location=signode)


class DBusInterface(DBusDescription):
    """
    Implementation of ``dbus:interface``.
    """

    def get_index_text(self, ifacename: str, name: str) -> str:
        return ifacename

    def before_content(self) -> None:
        self.env.ref_context["dbus:interface"] = self.arguments[0]

    def after_content(self) -> None:
        self.env.ref_context.pop("dbus:interface")

    def handle_signature(self, sig: str, signode: desc_signature) -> str:
        signode += addnodes.desc_annotation("interface ", "interface ")
        signode += addnodes.desc_name(sig, sig)
        return sig

    def run(self) -> List[Node]:
        _, node = super().run()
        name = self.arguments[0]
        section = nodes.section(ids=[name + "-section"])
        section += nodes.title(name, "%s interface" % name)
        section += node
        return [self.indexnode, section]


class DBusMember(DBusDescription):

    signal = False


class DBusMethod(DBusMember):
    """
    Implementation of ``dbus:method``.
    """

    option_spec: OptionSpec = DBusMember.option_spec.copy()
    option_spec.update(
        {
            "noreply": directives.flag,
        }
    )

    doc_field_types: List[Field] = [
        TypedField(
            "arg",
            label=_("Arguments"),
            names=("arg",),
            rolename="arg",
            typerolename=None,
            typenames=("argtype", "type"),
        ),
        TypedField(
            "ret",
            label=_("Returns"),
            names=("ret",),
            rolename="ret",
            typerolename=None,
            typenames=("rettype", "type"),
        ),
    ]

    def get_index_text(self, ifacename: str, name: str) -> str:
        return _("%s() (%s method)") % (name, ifacename)

    def handle_signature(self, sig: str, signode: desc_signature) -> str:
        params = addnodes.desc_parameterlist()
        returns = addnodes.desc_parameterlist()

        contentnode = addnodes.desc_content()
        self.state.nested_parse(self.content, self.content_offset, contentnode)
        for child in contentnode:
            if isinstance(child, nodes.field_list):
                for field in child:
                    ty, sg, name = field[0].astext().split(None, 2)
                    param = addnodes.desc_parameter()
                    param += addnodes.desc_sig_keyword_type(sg, sg)
                    param += addnodes.desc_sig_space()
                    param += addnodes.desc_sig_name(name, name)
                    if ty == "arg":
                        params += param
                    elif ty == "ret":
                        returns += param

        anno = "signal " if self.signal else "method "
        signode += addnodes.desc_annotation(anno, anno)
        signode += addnodes.desc_name(sig, sig)
        signode += params
        if not self.signal and "noreply" not in self.options:
            ret = addnodes.desc_returns()
            ret += returns
            signode += ret

        return sig


class DBusSignal(DBusMethod):
    """
    Implementation of ``dbus:signal``.
    """

    doc_field_types: List[Field] = [
        TypedField(
            "arg",
            label=_("Arguments"),
            names=("arg",),
            rolename="arg",
            typerolename=None,
            typenames=("argtype", "type"),
        ),
    ]
    signal = True

    def get_index_text(self, ifacename: str, name: str) -> str:
        return _("%s() (%s signal)") % (name, ifacename)


class DBusProperty(DBusMember):
    """
    Implementation of ``dbus:property``.
    """

    option_spec: OptionSpec = DBusMember.option_spec.copy()
    option_spec.update(
        {
            "type": directives.unchanged,
            "readonly": directives.flag,
            "writeonly": directives.flag,
            "readwrite": directives.flag,
            "emits-changed": directives.unchanged,
        }
    )

    doc_field_types: List[Field] = []

    def get_index_text(self, ifacename: str, name: str) -> str:
        return _("%s (%s property)") % (name, ifacename)

    def transform_content(self, contentnode: addnodes.desc_content) -> None:
        fieldlist = nodes.field_list()
        access = None
        if "readonly" in self.options:
            access = _("read-only")
        if "writeonly" in self.options:
            access = _("write-only")
        if "readwrite" in self.options:
            access = _("read & write")
        if access:
            content = nodes.Text(access)
            fieldname = nodes.field_name("", _("Access"))
            fieldbody = nodes.field_body("", nodes.paragraph("", "", content))
            field = nodes.field("", fieldname, fieldbody)
            fieldlist += field
        emits = self.options.get("emits-changed", None)
        if emits:
            content = nodes.Text(emits)
            fieldname = nodes.field_name("", _("Emits Changed"))
            fieldbody = nodes.field_body("", nodes.paragraph("", "", content))
            field = nodes.field("", fieldname, fieldbody)
            fieldlist += field
        if len(fieldlist) > 0:
            contentnode.insert(0, fieldlist)

    def handle_signature(self, sig: str, signode: desc_signature) -> str:
        contentnode = addnodes.desc_content()
        self.state.nested_parse(self.content, self.content_offset, contentnode)
        ty = self.options.get("type")

        signode += addnodes.desc_annotation("property ", "property ")
        signode += addnodes.desc_name(sig, sig)
        signode += addnodes.desc_sig_punctuation("", ":")
        signode += addnodes.desc_sig_keyword_type(ty, ty)
        return sig

    def run(self) -> List[Node]:
        self.name = "dbus:member"
        return super().run()


class DBusXRef(XRefRole):
    def process_link(self, env, refnode, has_explicit_title, title, target):
        refnode["dbus:interface"] = env.ref_context.get("dbus:interface")
        if not has_explicit_title:
            title = title.lstrip(".")  # only has a meaning for the target
            target = target.lstrip("~")  # only has a meaning for the title
            # if the first character is a tilde, don't display the module/class
            # parts of the contents
            if title[0:1] == "~":
                title = title[1:]
                dot = title.rfind(".")
                if dot != -1:
                    title = title[dot + 1:]
        # if the first character is a dot, search more specific namespaces first
        # else search builtins first
        if target[0:1] == ".":
            target = target[1:]
            refnode["refspecific"] = True
        return title, target


class DBusIndex(Index):
    """
    Index subclass to provide a D-Bus interfaces index.
    """

    name = "dbusindex"
    localname = _("D-Bus Interfaces Index")
    shortname = _("dbus")

    def generate(
        self, docnames: Iterable[str] = None
    ) -> Tuple[List[Tuple[str, List[IndexEntry]]], bool]:
        content: Dict[str, List[IndexEntry]] = {}
        # list of prefixes to ignore
        ignores: List[str] = self.domain.env.config["dbus_index_common_prefix"]
        ignores = sorted(ignores, key=len, reverse=True)

        ifaces = sorted(
            [
                x
                for x in self.domain.data["objects"].items()
                if x[1].objtype == "interface"
            ],
            key=lambda x: x[0].lower(),
        )
        for name, (docname, node_id, _) in ifaces:
            if docnames and docname not in docnames:
                continue

            for ignore in ignores:
                if name.startswith(ignore):
                    name = name[len(ignore):]
                    stripped = ignore
                    break
            else:
                stripped = ""

            entries = content.setdefault(name[0].lower(), [])
            entries.append(IndexEntry(stripped + name, 0,
                           docname, node_id, "", "", ""))

        # sort by first letter
        sorted_content = sorted(content.items())

        return sorted_content, False


class ObjectEntry(NamedTuple):
    docname: str
    node_id: str
    objtype: str


class DBusDomain(Domain):
    """
    Implementation of the D-Bus domain.
    """

    name = "dbus"
    label = "D-Bus"
    object_types: Dict[str, ObjType] = {
        "interface": ObjType(_("interface"), "iface", "obj"),
        "method": ObjType(_("method"), "meth", "obj"),
        "signal": ObjType(_("signal"), "sig", "obj"),
        "property": ObjType(_("property"), "attr", "_prop", "obj"),
    }
    directives = {
        "interface": DBusInterface,
        "method": DBusMethod,
        "signal": DBusSignal,
        "property": DBusProperty,
    }
    roles = {
        "iface": DBusXRef(),
        "meth": DBusXRef(),
        "sig": DBusXRef(),
        "prop": DBusXRef(),
    }
    initial_data: Dict[str, Dict[str, Tuple[Any]]] = {
        "objects": {},  # fullname -> ObjectEntry
    }
    indices = [
        DBusIndex,
    ]

    @property
    def objects(self) -> Dict[str, ObjectEntry]:
        return self.data.setdefault("objects", {})  # fullname -> ObjectEntry

    def note_object(
        self, name: str, objtype: str, node_id: str, location: Any = None
    ) -> None:
        self.objects[name] = ObjectEntry(self.env.docname, node_id, objtype)

    def clear_doc(self, docname: str) -> None:
        for fullname, obj in list(self.objects.items()):
            if obj.docname == docname:
                del self.objects[fullname]

    def find_obj(self, typ: str, name: str) -> Optional[Tuple[str, ObjectEntry]]:
        # skip parens
        if name[-2:] == "()":
            name = name[:-2]
        if typ in ("meth", "sig", "prop"):
            try:
                ifacename, name = name.rsplit(".", 1)
            except ValueError:
                pass
        return self.objects.get(name)

    def resolve_xref(
        self,
        env: "BuildEnvironment",
        fromdocname: str,
        builder: "Builder",
        typ: str,
        target: str,
        node: pending_xref,
        contnode: Element,
    ) -> Optional[Element]:
        """Resolve the pending_xref *node* with the given *typ* and *target*."""
        objdef = self.find_obj(typ, target)
        if objdef:
            return node_utils.make_refnode(
                builder, fromdocname, objdef.docname, objdef.node_id, contnode
            )

    def get_objects(self) -> Iterator[Tuple[str, str, str, str, str, int]]:
        for refname, obj in self.objects.items():
            yield (refname, refname, obj.objtype, obj.docname, obj.node_id, 1)


def setup(app):
    app.add_domain(DBusDomain)
    app.add_config_value("dbus_index_common_prefix", [], "env")



# Create a package query.
packages = libdnf5.rpm.PackageQuery(base)

# Filter the packages, the filters can be stacked one after another.
packages.filter_name(["one"])

# Iterate over the filtered packages in the query.
for pkg in packages:
    print(pkg.get_name())



#!/usr/bin/python3


import hashlib
import os
import xml.etree.ElementTree


TOPDIR = os.path.dirname(os.path.abspath(__file__))

NAMESPACES = {
    "": "http://linux.duke.edu/metadata/repo",
    "rpm": "http://linux.duke.edu/metadata/rpm",
}


for key, value in NAMESPACES.items():
    xml.etree.ElementTree.register_namespace(key, value)


for repoid in os.listdir(TOPDIR):
    repo_dir = os.path.join(TOPDIR, repoid)

    if not os.path.isdir(repo_dir):
        continue

    print("Updating checksums in repo {}...".format(repoid))

    repomd_xml = os.path.join(repo_dir, "repodata", "repomd.xml")
    root = xml.etree.ElementTree.parse(repomd_xml)

    # iterate through all <data type="..."> nodes
    data_nodes = root.findall('./data', namespaces=NAMESPACES)
    for data in data_nodes:
        # determine indexed file location
        location = data.find(
            "./location", namespaces=NAMESPACES).attrib["href"]
        location = os.path.join(repo_dir, location)

        # compute and store the current checksum
        checksum = data.find("./checksum", namespaces=NAMESPACES)
        checksum_type = checksum.attrib["type"]
        hash = hashlib.new(checksum_type)
        hash.update(open(location, "rb").read())
        checksum.text = hash.hexdigest()

        # compute and store the current checksum of uncompressed file
        # it is equal to 'checksum' because we're not using any compression in order to keep data in plain text
        open_checksum = data.find("./open-checksum", namespaces=NAMESPACES)
        open_checksum_type = open_checksum.attrib["type"]
        open_hash = hashlib.new(checksum_type)
        open_hash.update(open(location, "rb").read())
        open_checksum.text = open_hash.hexdigest()

    root.write(repomd_xml)



import os
import sys
import subprocess

DIR = os.path.abspath(os.path.dirname(__file__))
# get parent directory path
SOURCE_DIR = os.path.dirname(DIR)


def configure(src_path, dst_path, substitutions):
    print("Configuring {}...".format(src_path))

    # paths are relative to setup.py location
    src_path = os.path.join(DIR, src_path)
    dst_path = os.path.join(DIR, dst_path)

    data = open(src_path, "r").read()
    for pattern, substitution in substitutions.items():
        data = data.replace(pattern, substitution)
    open(dst_path, "w").write(data)


def generate_bindings_from_dir(in_dir, out_dir):
    swig_options = ["-python", "-DSWIGWORDSIZE64", "-doxygen",
                    "-relativeimport", "-outdir", out_dir, "-c++"]
    swig_includes = ["-I" + os.path.join(SOURCE_DIR, "include"), "-I" + os.path.join(
        SOURCE_DIR, "common"), "-I/usr/include/python3.11", "-I" + SOURCE_DIR]

    for in_file in os.listdir(in_dir):
        # exclude shared.i which is not a standalone interface file
        # it is included in other files.
        if in_file.endswith(".i") and in_file != "shared.i":
            print("Generating bindings for: " + in_file)
            subprocess.run(["/usr/bin/swig"] + swig_options + ["-interface", "_" + in_file[:-2]
                                                               ] + swig_includes + [os.path.join(in_dir, in_file)], cwd=DIR, check=True)


def main():
    # configure the .in files
    configure("Doxyfile.in", "Doxyfile", {
              "@CMAKE_SOURCE_DIR@": SOURCE_DIR})
    configure("conf.py.in", "conf.py", {
              "@CMAKE_CURRENT_BINARY_DIR@": DIR, "@AUTOAPI_EXTENSION@": "\'autoapi.extension\',"})

    # In place replace autodoc directives with autoapidoc directives.
    # Our readthedocs workflow uses autoapidoc instead of autodoc because
    # it is able to parse code generated by swig without including the
    # module -> therefore there is no need for a full build in order to
    # generate python docs.
    for in_file in os.listdir(os.path.join(DIR, "api/python")):
        if in_file.endswith(".rst"):
            file = os.path.join("api/python/", in_file)
            configure(file, file, {"automodule": "autoapimodule"})

    # run doxygen manually
    print("Running doxygen...")
    subprocess.run(["doxygen"], cwd=DIR, check=True)

    # run swig manually to generate Python bindings which are then used to generate Python API docs
    print("Running SWIG...")

    # libdnf5
    # Generate bindings outside of doc dir, into their python bindings dir. This has to match with path provided to autoapi_dirs in conf.py.in
    generate_bindings_from_dir(os.path.join(
        SOURCE_DIR, "bindings/libdnf5"), os.path.join(SOURCE_DIR, "bindings/python3/libdnf5"))

    # libdnf5-cli
    generate_bindings_from_dir(os.path.join(SOURCE_DIR, "bindings/libdnf5_cli"),
                               os.path.join(SOURCE_DIR, "bindings/python3/libdnf5_cli"))


if __name__ == "__main__":
    if os.environ.get("READTHEDOCS") != "True":
        print("This script is supposed to be used only to build documentation in Read the Docs.")
        print("Use cmake && make doc instead to build documentation locally")
        sys.exit(1)
    main()



# Get or set configuration options.
config = base.get_config()
config.skip_unavailable = True
print(config.skip_unavailable)

# Get or set values of variables.
vars = base.get_vars()
vars.set("releasever", "42")
print(vars.get_value("releasever"))



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#!/usr/bin/python3

#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus
import os

DNFDAEMON_BUS_NAME = 'org.rpm.dnf.v0'
DNFDAEMON_OBJECT_PATH = '/' + DNFDAEMON_BUS_NAME.replace('.', '/')

IFACE_SESSION_MANAGER = '{}.SessionManager'.format(DNFDAEMON_BUS_NAME)
IFACE_REPO = '{}.rpm.Repo'.format(DNFDAEMON_BUS_NAME)
IFACE_REPOCONF = '{}.rpm.RepoConf'.format(DNFDAEMON_BUS_NAME)
IFACE_RPM = '{}.rpm.Rpm'.format(DNFDAEMON_BUS_NAME)
IFACE_GOAL = '{}.Goal'.format(DNFDAEMON_BUS_NAME)
IFACE_ADVISORY = '{}.Advisory'.format(DNFDAEMON_BUS_NAME)


bus = dbus.SystemBus()
iface_session = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, DNFDAEMON_OBJECT_PATH),
    dbus_interface=IFACE_SESSION_MANAGER)

session = iface_session.open_session(
    dbus.Dictionary({}, signature=dbus.Signature('sv')))

iface_rpm = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, session),
    dbus_interface=IFACE_RPM)
iface_advisory = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, session),
    dbus_interface=IFACE_ADVISORY)

# Information about the severity of the upgrade is stored in the updateinfo
# repository metadata and is not directly accessible from the list of possible
# upgrades. This means we first need to retrieve all advisories and then search
# them for severity for all potential upgrade packages.

# First get all available advisories, we are interested only in "severity" and
# "collections" (list of packages and modules) fields.
options = {
    "advisory_attrs": [
        # "advisoryid",
        # "name",
        # "title",
        # "type",
        "severity",
        # "status",
        # "vendor",
        # "description",
        # "buildtime",
        # "message",
        # "rights",
        "collections",
        # "references",
    ],
    "availability": "available",
}
advisory_list = iface_advisory.list(options)

# auxiliary dictionary to map package NEVRA to severity of advisory it belongs to
nevra_to_severity = dict()
for adv in advisory_list:
    severity = str(adv["severity"])
    for col in adv["collections"]:
        if "packages" in col:
            for pkg in col["packages"]:
                nevra = pkg["nevra"]
                nevra_to_severity[nevra] = severity


# retrieve potential upgrades and print the packages along with their
# respective severities
options = {
    "package_attrs": [
        "nevra",
        "repo_id",
    ],
    "scope": "upgrades",
    "latest-limit": 1,
}
upgrades = iface_rpm.list(options)
for pkg in upgrades:
    nevra = pkg["nevra"]
    severity = nevra_to_severity.get(nevra, "<unknown>")
    print("{} (@{}): {}".format(nevra, pkg["repo_id"], severity))



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#!/usr/bin/python3

#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

"""
This is an example how to use `list_fd()` method of dnf5daemon-server Rpm interface.
"""

import dbus
import json
import os
import select
import time

DNFDAEMON_BUS_NAME = 'org.rpm.dnf.v0'
DNFDAEMON_OBJECT_PATH = '/' + DNFDAEMON_BUS_NAME.replace('.', '/')

IFACE_SESSION_MANAGER = '{}.SessionManager'.format(DNFDAEMON_BUS_NAME)
IFACE_RPM = '{}.rpm.Rpm'.format(DNFDAEMON_BUS_NAME)


def repoquery(iface_rpm, options):
    """Generator function that yields packages as they arrive from the server."""

    # create a pipe and pass the write end to the server
    pipe_r, pipe_w = os.pipe()
    # transfer id serves as an identifier of the pipe transfer for a signal emitted
    # after server finish. This example does not use it.
    transfer_id = iface_rpm.list_fd(options, pipe_w)
    print("transfer_id: ", transfer_id)
    # close the write end - otherwise poll cannot detect the end of transmission
    os.close(pipe_w)

    # decoder that will be used to parse incoming data
    parser = json.JSONDecoder()

    # prepare for polling
    poller = select.poll()
    poller.register(pipe_r, select.POLLIN)
    # wait for data 10 secs at most
    timeout = 10000
    # 64k is a typical size of a pipe
    buffer_size = 65536

    # remaining string to parse (can contain unfinished json from previous run)
    to_parse = ""
    # remaining raw data (i.e. data before UTF decoding)
    raw_data = b""
    while True:
        # wait for data
        polled_event = poller.poll(timeout)
        if not polled_event:
            print("Timeout reached.")
            break

        # we know there is only one fd registered in poller
        descriptor, event = polled_event[0]
        # read a chunk of data
        buffer = os.read(descriptor, buffer_size)
        if not buffer:
            # end of file
            break

        raw_data += buffer
        try:
            to_parse += raw_data.decode()
            # decode successful, clear remaining raw data
            raw_data = b""
        except UnicodeDecodeError:
            # Buffer size split data in the middle of multibyte UTF character.
            # Need to read another chunk of data.
            continue

        # parse JSON objects from the string
        while to_parse:
            try:
                # skip all chars till begin of next JSON objects (new lines mostly)
                json_obj_start = to_parse.find('{')
                if json_obj_start < 0:
                    break
                obj, end = parser.raw_decode(to_parse[json_obj_start:])
                yield obj
                to_parse = to_parse[(json_obj_start+end):]
            except json.decoder.JSONDecodeError as e:
                # this is just example which assumes that every decode error
                # means the data are incomplete (buffer size split the json
                # object in the middle). So the handler does not do anything
                # just break the parsing cycle and continue polling.
                break

    # finally close read end of the pipe
    os.close(pipe_r)

    # non-empty raw_data here means the raw_data.decode() failed
    if raw_data:
        raise Exception("Failed to decode part of received data.")

    # non-empty to_parse here means there are some unfinished (or generally unparsable)
    # JSON objects in the stream.
    if to_parse.strip():
        raise Exception("Failed to parse part of received data.")


bus = dbus.SystemBus()

# open a new session with dnf5daemon-server
iface_session = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, DNFDAEMON_OBJECT_PATH),
    dbus_interface=IFACE_SESSION_MANAGER)
session = iface_session.open_session(
    dbus.Dictionary({}, signature=dbus.Signature('sv')))

options = {
    # retrieve all package attributes
    "package_attrs": [
        "name",
        "epoch",
        "version",
        "release",
        "arch",
        "repo_id",
        "from_repo_id",
        "is_installed",
        "install_size",
        "download_size",
        "buildtime",
        "sourcerpm",
        "summary",
        "url",
        "license",
        "description",
        "files",
        "changelogs",
        "provides",
        "requires",
        "requires_pre",
        "conflicts",
        "obsoletes",
        "recommends",
        "suggests",
        "enhances",
        "supplements",
        "evr",
        "nevra",
        "full_nevra",
        "reason",
        "vendor",
    ],
    # take all packages into account (other supported scopes are installed, available,
    # upgrades, upgradable)
    "scope": "all",
    # get only packages with name starting with "a"
    "patterns": ["a*"],
    # return only the latest version for each name.arch
    "latest-limit": 1,
}
iface_rpm = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, session),
    dbus_interface=IFACE_RPM)


i = 1
for pkg in repoquery(iface_rpm, options):
    print(i, pkg["nevra"])
    i += 1



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5

import base_test_case


class TestNevra(base_test_case.BaseTestCase):
    def test_nevra(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-8:3.6.9-11.fc100.x86_64", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEVRA))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish")
        self.assertEqual(nevra.get_epoch(), "8")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "11.fc100")
        self.assertEqual(nevra.get_arch(), "x86_64")

        # test that to_nevra_string() and to_full_nevra_string() template functions work
        self.assertEqual(
            libdnf5.rpm.to_nevra_string(nevra), "four-of-fish-8:3.6.9-11.fc100.x86_64")
        self.assertEqual(
            libdnf5.rpm.to_full_nevra_string(nevra), "four-of-fish-8:3.6.9-11.fc100.x86_64")

    def test_nevra_without_epoch(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-3.6.9-11.fc100.x86_64", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEVRA))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish")
        self.assertEqual(nevra.get_epoch(), "")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "11.fc100")
        self.assertEqual(nevra.get_arch(), "x86_64")

        # test that to_nevra_string() and to_full_nevra_string() template functions work without epoch
        self.assertEqual(
            libdnf5.rpm.to_nevra_string(nevra), "four-of-fish-3.6.9-11.fc100.x86_64")
        self.assertEqual(
            libdnf5.rpm.to_full_nevra_string(nevra), "four-of-fish-0:3.6.9-11.fc100.x86_64")

    def test_nevra_with_colon_in_name(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-f:ish-3.6.9-11.fc100.x86_64", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEVRA))
        self.assertEqual(len(nevras), 0)

    def test_nevra_with_two_colons(self):
        with self.assertRaises(libdnf5.exception.RpmNevraIncorrectInputError):
            libdnf5.rpm.Nevra.parse(
                "four-of-fish-8:9:3.6.9-11.fc100.x86_64", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEVRA))

    def test_nevra_form_NEV(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-8:3.6.9", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEV))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish")
        self.assertEqual(nevra.get_epoch(), "8")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "")
        self.assertEqual(nevra.get_arch(), "")

        nevras = libdnf5.rpm.Nevra.parse(
            "fish-8:3.6.9", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEV))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "fish")
        self.assertEqual(nevra.get_epoch(), "8")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "")
        self.assertEqual(nevra.get_arch(), "")

        nevras = libdnf5.rpm.Nevra.parse(
            "fish-3.6.9", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEV))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "fish")
        self.assertEqual(nevra.get_epoch(), "")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "")
        self.assertEqual(nevra.get_arch(), "")

        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-3.6.9.i686", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEV))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish")
        self.assertEqual(nevra.get_epoch(), "")
        self.assertEqual(nevra.get_version(), "3.6.9.i686")
        self.assertEqual(nevra.get_release(), "")
        self.assertEqual(nevra.get_arch(), "")

    def test_nevra_form_NA(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-3.6.9.i686", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish-3.6.9")
        self.assertEqual(nevra.get_epoch(), "")
        self.assertEqual(nevra.get_version(), "")
        self.assertEqual(nevra.get_release(), "")
        self.assertEqual(nevra.get_arch(), "i686")

    def test_nevra_incorrect_form_NA(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "name.ar-ch", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))
        self.assertEqual(len(nevras), 0)

        nevras = libdnf5.rpm.Nevra.parse(
            "name.-arch", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))
        self.assertEqual(len(nevras), 0)

        nevras = libdnf5.rpm.Nevra.parse(
            "name.", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))
        self.assertEqual(len(nevras), 0)

        nevras = libdnf5.rpm.Nevra.parse(
            "name", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))
        self.assertEqual(len(nevras), 0)

    def test_nevra_invalid_characters(self):
        with self.assertRaises(libdnf5.exception.RpmNevraIncorrectInputError):
            nevras = libdnf5.rpm.Nevra.parse(
                "four-of(fish.i686)", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))

    def test_nevra_with_glob_in_epoch(self):
        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-[01]:3.6.9-11.fc100.x86_64", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEVRA))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish")
        self.assertEqual(nevra.get_epoch(), "[01]")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "11.fc100")
        self.assertEqual(nevra.get_arch(), "x86_64")

        nevras = libdnf5.rpm.Nevra.parse(
            "four-of-fish-?:3.6.9-11.fc100.x86_64", libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NEVRA))
        self.assertEqual(len(nevras), 1)
        nevra = nevras.pop()
        self.assertEqual(nevra.get_name(), "four-of-fish")
        self.assertEqual(nevra.get_epoch(), "?")
        self.assertEqual(nevra.get_version(), "3.6.9")
        self.assertEqual(nevra.get_release(), "11.fc100")
        self.assertEqual(nevra.get_arch(), "x86_64")



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus

import support


class GoalTest(support.InstallrootCase):

    def test_do_transaction_before_resolve(self):
        with self.assertRaises(dbus.exceptions.DBusException) as cm:
            self.iface_goal.do_transaction(dbus.Dictionary({}, signature='sv'))
        self.assertEqual(str(
            cm.exception), 'org.rpm.dnf.v0.rpm.Rpm.TransactionError: Transaction has to be resolved first.')



repo_sack = base.get_repo_sack()

# Create a new repo with the given id.
#
# The repo is a weak pointer to an object owned by the repo sack.
repo = repo_sack.create_repo("rpm-repo1")

# Configure the repo.
#
# Setting at least one of the baseurl, mirrorlist or metalink options is
# mandatory.
#
# baseurl examples:
# * /absolute/path/
# * file:///absolute/path/url/
# * https://example.com/url/
repo.get_config().baseurl = baseurl

# If out of date, downloads fresh metadata of all available repositories and
# loads the repositories into memory.
#
# `libdnf5.repo.Repo.Type_AVAILABLE` as first argument says to load only the available
# repositories (repository SYSTEM, that contains installed packages, is not loaded).
repo_sack.load_repos(libdnf5.repo.Repo.Type_AVAILABLE)



# Copyright Contributors to the DNF5 project.
# Copyright (C) 2008-2011 Red Hat, Inc.
# Author: David Zeuthen <davidz@redhat.com>
# SPDX-License-Identifier: LGPL-2.1-or-later
# Based from "GDBus - GLib D-Bus Library":
#

#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General
# Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
#

import xml.parsers.expat


class Annotation:
    def __init__(self, key, value):
        self.key = key
        self.value = value
        self.annotations = []
        self.since = ""


class Arg:
    def __init__(self, name, signature):
        self.name = name
        self.signature = signature
        self.annotations = []
        self.doc_string = ""
        self.since = ""


class Method:
    def __init__(self, name, h_type_implies_unix_fd=True):
        self.name = name
        self.h_type_implies_unix_fd = h_type_implies_unix_fd
        self.in_args = []
        self.out_args = []
        self.annotations = []
        self.doc_string = ""
        self.since = ""
        self.deprecated = False
        self.unix_fd = False


class Signal:
    def __init__(self, name):
        self.name = name
        self.args = []
        self.annotations = []
        self.doc_string = ""
        self.since = ""
        self.deprecated = False


class Property:
    def __init__(self, name, signature, access):
        self.name = name
        self.signature = signature
        self.access = access
        self.annotations = []
        self.arg = Arg("value", self.signature)
        self.arg.annotations = self.annotations
        self.readable = False
        self.writable = False
        if self.access == "readwrite":
            self.readable = True
            self.writable = True
        elif self.access == "read":
            self.readable = True
        elif self.access == "write":
            self.writable = True
        else:
            raise ValueError('Invalid access type "{}"'.format(self.access))
        self.doc_string = ""
        self.since = ""
        self.deprecated = False
        self.emits_changed_signal = True


class Interface:
    def __init__(self, name):
        self.name = name
        self.methods = []
        self.signals = []
        self.properties = []
        self.annotations = []
        self.doc_string = ""
        self.doc_string_brief = ""
        self.since = ""
        self.deprecated = False


class DBusXMLParser:
    STATE_TOP = "top"
    STATE_NODE = "node"
    STATE_INTERFACE = "interface"
    STATE_METHOD = "method"
    STATE_SIGNAL = "signal"
    STATE_PROPERTY = "property"
    STATE_ARG = "arg"
    STATE_ANNOTATION = "annotation"
    STATE_IGNORED = "ignored"

    def __init__(self, xml_data, h_type_implies_unix_fd=True):
        self._parser = xml.parsers.expat.ParserCreate()
        self._parser.CommentHandler = self.handle_comment
        self._parser.CharacterDataHandler = self.handle_char_data
        self._parser.StartElementHandler = self.handle_start_element
        self._parser.EndElementHandler = self.handle_end_element

        self.parsed_interfaces = []
        self._cur_object = None

        self.state = DBusXMLParser.STATE_TOP
        self.state_stack = []
        self._cur_object = None
        self._cur_object_stack = []

        self.doc_comment_last_symbol = ""

        self._h_type_implies_unix_fd = h_type_implies_unix_fd

        self._parser.Parse(xml_data)

    COMMENT_STATE_BEGIN = "begin"
    COMMENT_STATE_PARAMS = "params"
    COMMENT_STATE_BODY = "body"
    COMMENT_STATE_SKIP = "skip"

    def handle_comment(self, data):
        comment_state = DBusXMLParser.COMMENT_STATE_BEGIN
        lines = data.split("\n")
        symbol = ""
        body = ""
        in_para = False
        params = {}
        for line in lines:
            orig_line = line
            line = line.lstrip()
            if comment_state == DBusXMLParser.COMMENT_STATE_BEGIN:
                if len(line) > 0:
                    colon_index = line.find(": ")
                    if colon_index == -1:
                        if line.endswith(":"):
                            symbol = line[0: len(line) - 1]
                            comment_state = DBusXMLParser.COMMENT_STATE_PARAMS
                        else:
                            comment_state = DBusXMLParser.COMMENT_STATE_SKIP
                    else:
                        symbol = line[0:colon_index]
                        rest_of_line = line[colon_index + 2:].strip()
                        if len(rest_of_line) > 0:
                            body += rest_of_line + "\n"
                        comment_state = DBusXMLParser.COMMENT_STATE_PARAMS
            elif comment_state == DBusXMLParser.COMMENT_STATE_PARAMS:
                if line.startswith("@"):
                    colon_index = line.find(": ")
                    if colon_index == -1:
                        comment_state = DBusXMLParser.COMMENT_STATE_BODY
                        if not in_para:
                            in_para = True
                        body += orig_line + "\n"
                    else:
                        param = line[1:colon_index]
                        docs = line[colon_index + 2:]
                        params[param] = docs
                else:
                    comment_state = DBusXMLParser.COMMENT_STATE_BODY
                    if len(line) > 0:
                        if not in_para:
                            in_para = True
                        body += orig_line + "\n"
            elif comment_state == DBusXMLParser.COMMENT_STATE_BODY:
                if len(line) > 0:
                    if not in_para:
                        in_para = True
                    body += orig_line + "\n"
                else:
                    if in_para:
                        body += "\n"
                        in_para = False
        if in_para:
            body += "\n"

        if symbol != "":
            self.doc_comment_last_symbol = symbol
            self.doc_comment_params = params
            self.doc_comment_body = body

    def handle_char_data(self, data):
        # print 'char_data=%s'%data
        pass

    def handle_start_element(self, name, attrs):
        old_state = self.state
        old_cur_object = self._cur_object
        if self.state == DBusXMLParser.STATE_IGNORED:
            self.state = DBusXMLParser.STATE_IGNORED
        elif self.state == DBusXMLParser.STATE_TOP:
            if name == DBusXMLParser.STATE_NODE:
                self.state = DBusXMLParser.STATE_NODE
            else:
                self.state = DBusXMLParser.STATE_IGNORED
        elif self.state == DBusXMLParser.STATE_NODE:
            if name == DBusXMLParser.STATE_INTERFACE:
                self.state = DBusXMLParser.STATE_INTERFACE
                iface = Interface(attrs["name"])
                self._cur_object = iface
                self.parsed_interfaces.append(iface)
            elif name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

            # assign docs, if any
            if "name" in attrs and self.doc_comment_last_symbol == attrs["name"]:
                self._cur_object.doc_string = self.doc_comment_body
                if "short_description" in self.doc_comment_params:
                    short_description = self.doc_comment_params["short_description"]
                    self._cur_object.doc_string_brief = short_description
                if "since" in self.doc_comment_params:
                    self._cur_object.since = self.doc_comment_params["since"].strip(
                    )

        elif self.state == DBusXMLParser.STATE_INTERFACE:
            if name == DBusXMLParser.STATE_METHOD:
                self.state = DBusXMLParser.STATE_METHOD
                method = Method(
                    attrs["name"], h_type_implies_unix_fd=self._h_type_implies_unix_fd
                )
                self._cur_object.methods.append(method)
                self._cur_object = method
            elif name == DBusXMLParser.STATE_SIGNAL:
                self.state = DBusXMLParser.STATE_SIGNAL
                signal = Signal(attrs["name"])
                self._cur_object.signals.append(signal)
                self._cur_object = signal
            elif name == DBusXMLParser.STATE_PROPERTY:
                self.state = DBusXMLParser.STATE_PROPERTY
                prop = Property(attrs["name"], attrs["type"], attrs["access"])
                self._cur_object.properties.append(prop)
                self._cur_object = prop
            elif name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

            # assign docs, if any
            if "name" in attrs and self.doc_comment_last_symbol == attrs["name"]:
                self._cur_object.doc_string = self.doc_comment_body
                if "since" in self.doc_comment_params:
                    self._cur_object.since = self.doc_comment_params["since"].strip(
                    )

        elif self.state == DBusXMLParser.STATE_METHOD:
            if name == DBusXMLParser.STATE_ARG:
                self.state = DBusXMLParser.STATE_ARG
                arg_name = None
                if "name" in attrs:
                    arg_name = attrs["name"]
                arg = Arg(arg_name, attrs["type"])
                direction = attrs.get("direction", "in")
                if direction == "in":
                    self._cur_object.in_args.append(arg)
                elif direction == "out":
                    self._cur_object.out_args.append(arg)
                else:
                    raise ValueError(
                        'Invalid direction "{}"'.format(direction))
                self._cur_object = arg
            elif name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

            # assign docs, if any
            if self.doc_comment_last_symbol == old_cur_object.name:
                if "name" in attrs and attrs["name"] in self.doc_comment_params:
                    doc_string = self.doc_comment_params[attrs["name"]]
                    if doc_string is not None:
                        self._cur_object.doc_string = doc_string
                    if "since" in self.doc_comment_params:
                        self._cur_object.since = self.doc_comment_params[
                            "since"
                        ].strip()

        elif self.state == DBusXMLParser.STATE_SIGNAL:
            if name == DBusXMLParser.STATE_ARG:
                self.state = DBusXMLParser.STATE_ARG
                arg_name = None
                if "name" in attrs:
                    arg_name = attrs["name"]
                arg = Arg(arg_name, attrs["type"])
                self._cur_object.args.append(arg)
                self._cur_object = arg
            elif name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

            # assign docs, if any
            if self.doc_comment_last_symbol == old_cur_object.name:
                if "name" in attrs and attrs["name"] in self.doc_comment_params:
                    doc_string = self.doc_comment_params[attrs["name"]]
                    if doc_string is not None:
                        self._cur_object.doc_string = doc_string
                    if "since" in self.doc_comment_params:
                        self._cur_object.since = self.doc_comment_params[
                            "since"
                        ].strip()

        elif self.state == DBusXMLParser.STATE_PROPERTY:
            if name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

        elif self.state == DBusXMLParser.STATE_ARG:
            if name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

        elif self.state == DBusXMLParser.STATE_ANNOTATION:
            if name == DBusXMLParser.STATE_ANNOTATION:
                self.state = DBusXMLParser.STATE_ANNOTATION
                anno = Annotation(attrs["name"], attrs["value"])
                self._cur_object.annotations.append(anno)
                self._cur_object = anno
            else:
                self.state = DBusXMLParser.STATE_IGNORED

        else:
            raise ValueError(
                'Unhandled state "{}" while entering element with name "{}"'.format(
                    self.state, name
                )
            )

        self.state_stack.append(old_state)
        self._cur_object_stack.append(old_cur_object)

    def handle_end_element(self, name):
        self.state = self.state_stack.pop()
        self._cur_object = self._cur_object_stack.pop()


def parse_dbus_xml(xml_data):
    parser = DBusXMLParser(xml_data, True)
    return parser.parsed_interfaces



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5


class TestExceptions(unittest.TestCase):
    # List of all exceptions
    EXCEPTIONS = (
        libdnf5.exception.BaseTransactionError,
        libdnf5.exception.BaseTransactionErrorNested,

        libdnf5.exception.CompsInvalidPackageType,
        libdnf5.exception.CompsInvalidPackageTypeNested,

        #        libdnf5.module.NoModuleError,
        #        libdnf5.module.NoStreamError,
        #        libdnf5.module.EnabledStreamError,
        #        libdnf5.module.EnableMultipleStreamsError,
        #        libdnf5.module.ModuleConflictError,
        #        libdnf5.module.ModuleResolveError,
        #        libdnf5.module.ModuleError,

        #        libdnf5.module.InvalidModuleStatus,

        libdnf5.exception.RepoFileDownloadError,
        libdnf5.exception.RepoFileDownloadErrorNested,
        libdnf5.exception.RepoPackageDownloadError,
        libdnf5.exception.RepoPackageDownloadErrorNested,
        libdnf5.exception.RepoCacheError,
        libdnf5.exception.RepoCacheErrorNested,

        libdnf5.exception.RepoCacheonlyError,
        libdnf5.exception.RepoCacheonlyErrorNested,
        libdnf5.exception.RepoDownloadError,
        libdnf5.exception.RepoDownloadErrorNested,
        libdnf5.exception.RepoPgpError,
        libdnf5.exception.RepoPgpErrorNested,
        libdnf5.exception.RepoRpmError,
        libdnf5.exception.RepoRpmErrorNested,
        libdnf5.exception.RepoCompsError,
        libdnf5.exception.RepoCompsErrorNested,
        libdnf5.exception.RepoIdAlreadyExistsError,
        libdnf5.exception.RepoIdAlreadyExistsErrorNested,
        libdnf5.exception.RepoError,
        libdnf5.exception.RepoErrorNested,

        libdnf5.exception.RpmNevraIncorrectInputError,
        libdnf5.exception.RpmNevraIncorrectInputErrorNested,
        libdnf5.exception.RpmSignatureCheckError,
        libdnf5.exception.RpmSignatureCheckErrorNested,
        libdnf5.exception.RpmKeyImportError,
        libdnf5.exception.RpmKeyImportErrorNested,

        libdnf5.exception.TransactionInvalidTransactionItemAction,
        libdnf5.exception.TransactionInvalidTransactionItemActionNested,
        libdnf5.exception.TransactionInvalidTransactionItemReason,
        libdnf5.exception.TransactionInvalidTransactionItemReasonNested,
        libdnf5.exception.TransactionInvalidTransactionItemState,
        libdnf5.exception.TransactionInvalidTransactionItemStateNested,
        libdnf5.exception.TransactionInvalidTransactionItemType,
        libdnf5.exception.TransactionInvalidTransactionItemTypeNested,
        libdnf5.exception.TransactionInvalidTransactionState,
        libdnf5.exception.TransactionInvalidTransactionStateNested,

        libdnf5.exception.InaccessibleConfigError,
        libdnf5.exception.InaccessibleConfigErrorNested,
        libdnf5.exception.MissingConfigError,
        libdnf5.exception.MissingConfigErrorNested,
        libdnf5.exception.InvalidConfigError,
        libdnf5.exception.InvalidConfigErrorNested,

        libdnf5.exception.ConfigParserSectionNotFoundError,
        libdnf5.exception.ConfigParserSectionNotFoundErrorNested,
        libdnf5.exception.ConfigParserOptionNotFoundError,
        libdnf5.exception.ConfigParserOptionNotFoundErrorNested,
        libdnf5.exception.ConfigParserError,
        libdnf5.exception.ConfigParserErrorNested,

        libdnf5.exception.OptionValueNotSetError,
        libdnf5.exception.OptionValueNotSetErrorNested,
        libdnf5.exception.OptionValueNotAllowedError,
        libdnf5.exception.OptionValueNotAllowedErrorNested,
        libdnf5.exception.OptionInvalidValueError,
        libdnf5.exception.OptionInvalidValueErrorNested,
        libdnf5.exception.OptionError,
        libdnf5.exception.OptionErrorNested,

        libdnf5.exception.OptionBindsOptionNotFoundError,
        libdnf5.exception.OptionBindsOptionNotFoundErrorNested,
        libdnf5.exception.OptionBindsOptionAlreadyExistsError,
        libdnf5.exception.OptionBindsOptionAlreadyExistsErrorNested,
        libdnf5.exception.OptionBindsError,
        libdnf5.exception.OptionBindsErrorNested,

        libdnf5.exception.ReadOnlyVariableError,
        libdnf5.exception.ReadOnlyVariableErrorNested,

        libdnf5.exception.FileSystemError,
        libdnf5.exception.FileSystemErrorNested,
        libdnf5.exception.SystemError,
        libdnf5.exception.SystemErrorNested,
        libdnf5.exception.RuntimeError,
        libdnf5.exception.RuntimeErrorNested,
        libdnf5.exception.Error,
        libdnf5.exception.ErrorNested,
        libdnf5.exception.UserAssertionError,
        libdnf5.exception.UserAssertionErrorNested,
        libdnf5.exception.AssertionError,
        libdnf5.exception.AssertionErrorNested,

        libdnf5.exception.NonLibdnf5Exception,
        libdnf5.exception.NonLibdnf5ExceptionNested
    )

    def test_all_exceptions_without_ctor(self):
        # Check that exceptions do not have a defined constructor.
        for ex in self.EXCEPTIONS:
            with self.assertRaises(AttributeError, msg=str(ex)) as e:
                raise ex()
            self.assertEqual(str(e.exception),
                             'No constructor defined', str(ex))



import libdnf5

# Create a new Base object.
base = libdnf5.base.Base()

# Set configuration for the tests
base_config = base.get_config()
base_config.cachedir = cachedir
base_config.installroot = installroot
# For unit tests only - Prevent loading plugins from the host system.
# In production code, plugins are typically left enabled (default behavior).
base_config.plugins = False

# Optionally, load configuration from the file defined in the current
# configuration and files in the drop-in directories.
base.load_config()

# Load variables and do other initialization based on the configuration.
base.setup()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import libdnf5

import base_test_case


class TestGlobalLogger(base_test_case.BaseTestCase):
    def test_creating_global_logger_again(self):
        # Try creating two instances of global logger
        router = libdnf5.logger.LogRouter()
        global_logger = libdnf5.logger.GlobalLogger()
        global_logger.set(router, libdnf5.logger.Logger.Level_DEBUG)
        with self.assertRaisesRegex(libdnf5.exception.UserAssertionError,
                                    'libdnf5/logger/global_logger.cpp:[0-9]+: libdnf5::GlobalLogger::GlobalLogger\\(\\):'
                                    ' API Assertion \'librepo_logger == nullptr\' failed:'
                                    ' Only one GlobalLogger can exist at a time'):
            libdnf5.logger.GlobalLogger()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5

import base_test_case


class TestPackageQuery(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.add_repo_repomd("repomd-repo1")

    def test_size(self):
        query = libdnf5.rpm.PackageQuery(self.base)
        self.assertEqual(query.size(), 3)

    def test_iterate_package_query(self):
        query = libdnf5.rpm.PackageQuery(self.base)

        # Iterates over reference "query".
        prev_id = 0
        for pkg in query:
            id = pkg.get_id().id
            self.assertGreater(id, prev_id)
            prev_id = id
        self.assertLess(prev_id, self.package_sack.get_nsolvables())
        self.assertGreaterEqual(prev_id, query.size())

        # Similar to above, but longer notation.
        query_iterator = iter(query)
        prev_id = 0
        while True:
            try:
                pkg = next(query_iterator)
                id = pkg.get_id().id
                self.assertGreater(id, prev_id)
                prev_id = id
            except StopIteration:
                break
        self.assertLess(prev_id, self.package_sack.get_nsolvables())
        self.assertGreaterEqual(prev_id, query.size())

    def test_iterate_package_query2(self):
        # Tests the iteration of an unreferenced PackageQuery object. The iterator must hold a reference
        # to the iterated object, otherwise the gargabe collector can remove the object.

        # Iterates directly over "libdnf5.rpm.PackageQuery(self.base)" result. No helping reference.
        prev_id = 0
        for pkg in libdnf5.rpm.PackageQuery(self.base):
            id = pkg.get_id().id
            self.assertGreater(id, prev_id)
            prev_id = id
        self.assertLess(prev_id, self.package_sack.get_nsolvables())
        self.assertGreaterEqual(
            prev_id, libdnf5.rpm.PackageQuery(self.base).size())

        # Another test. The iterator is created from the "query" reference, but the reference
        # is removed (set to "None") before starting the iteration.
        query = libdnf5.rpm.PackageQuery(self.base)
        query_iterator = iter(query)
        query = None
        prev_id = 0
        while True:
            try:
                pkg = next(query_iterator)
                id = pkg.get_id().id
                self.assertGreater(id, prev_id)
                prev_id = id
            except StopIteration:
                break
        self.assertLess(prev_id, self.package_sack.get_nsolvables())
        self.assertGreaterEqual(
            prev_id, libdnf5.rpm.PackageQuery(self.base).size())

    def test_filter_name(self):
        # Test QueryCmp::EQ
        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["pkg"])
        self.assertEqual(query.size(), 1)
        # TODO(dmach): implement __str__()
        self.assertEqual([i.get_nevra() for i in query], ["pkg-1.2-3.x86_64"])

        # ---

        # Test QueryCmp::GLOB
        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["pk*"], libdnf5.common.QueryCmp_GLOB)
        self.assertEqual(query.size(), 2)
        # TODO(dmach): implement __str__()
        self.assertEqual([i.get_nevra() for i in query], [
                         "pkg-1.2-3.x86_64", "pkg-libs-1:1.3-4.x86_64"])

    def test_resolve_pkg_spec(self):
        # Test passing empty forms
        query = libdnf5.rpm.PackageQuery(self.base)
        settings = libdnf5.base.ResolveSpecSettings()

        match, nevra = query.resolve_pkg_spec("pkg*", settings, True)

        self.assertEqual(query.size(), 2)
        self.assertEqual([i.get_nevra() for i in query], [
                         "pkg-1.2-3.x86_64", "pkg-libs-1:1.3-4.x86_64"])
        self.assertTrue(match)
        self.assertEqual(nevra.get_name(), "pkg*")
        self.assertTrue(nevra.has_just_name())

        # Test passing an explicit list of forms
        query = libdnf5.rpm.PackageQuery(self.base)
        settings = libdnf5.base.ResolveSpecSettings()
        settings.set_nevra_forms(
            libdnf5.rpm.VectorNevraForm(1, libdnf5.rpm.Nevra.Form_NA))

        match, nevra = query.resolve_pkg_spec("pkg.x86_64", settings, True)

        self.assertEqual(query.size(), 1)
        self.assertEqual([i.get_nevra() for i in query], ["pkg-1.2-3.x86_64"])
        self.assertTrue(match)
        self.assertEqual(nevra.get_name(), "pkg")
        self.assertEqual(nevra.get_arch(), "x86_64")
        self.assertFalse(nevra.has_just_name())

    def test_pkg_reason_value(self):
        # Test wrapper for reason enum
        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["pkg"])
        package = next(iter(query))
        self.assertEqual(package.get_reason(),
                         libdnf5.transaction.TransactionItemReason_NONE)

    def test_pkg_query_without_setup(self):
        # Create a new Base object
        base = libdnf5.base.Base()

        # Try to create a package query without running base.setup()
        with self.assertRaisesRegex(libdnf5.exception.UserAssertionError,
                                    'base/base_impl.hpp:[0-9]+: libdnf5::solv::RpmPool& libdnf5::Base::Impl::get_rpm_pool\\(\\):'
                                    ' API Assertion \'pool\' failed:'
                                    ' Base instance was not fully initialized by Base::setup()'):
            libdnf5.rpm.PackageQuery(base)

    def test_pkg_get_changelogs(self):
        # Test wrapper for changelogs vector
        query = libdnf5.rpm.PackageQuery(self.base)
        query.filter_name(["pkg"])
        package = next(iter(query))
        logs = package.get_changelogs()
        self.assertEqual(2, logs.size())
        log = next(iter(logs))
        self.assertEqual('First change', log.get_text())
        self.assertEqual('Joe Black', log.get_author())
        self.assertEqual(1641027600, log.get_timestamp())



import unittest

import libdnf5_cli

# See https://discuss.python.org/t/unittest-fail-if-zero-tests-were-discovered/21498, https://github.com/pytest-dev/pytest/issues/2393


class TestPlaceholder(unittest.TestCase):
    def test_placeholder(self):
        pass



# Create a goal.
goal = libdnf5.base.Goal(base)

# Add an RPM package named "one" for installation into the goal.
goal.add_rpm_install("one")

# Resolve the goal, create a transaction object.
transaction = goal.resolve()

if transaction.get_problems() != libdnf5.base.GoalProblem_NO_PROBLEM:
    for message in transaction.get_resolve_logs_as_strings():
        print(message)

# We can iterate over the resolved transaction and inspect the packages.
for tspkg in transaction.get_transaction_packages():
    print(
        tspkg.get_package().get_nevra(), ": ",
        libdnf5.base.transaction.transaction_item_action_to_string(
            tspkg.get_action()
        )
    )

# Download the packages.
transaction.download()

# Run the transaction.
transaction.run()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus
import unittest

import support


class SessionTest(unittest.TestCase):

    def setUp(self):
        self.bus = dbus.SystemBus()
        self.proxy = self.bus.get_object(support.DNFDAEMON_BUS_NAME,
                                         support.DNFDAEMON_OBJECT_PATH)
        self.iface = dbus.Interface(self.proxy,
                                    dbus_interface=support.IFACE_SESSION_MANAGER)

    def tearDown(self):
        pass

    def test_session(self):
        session = self.iface.open_session({})
        # session address has expected format
        self.assertRegex(
            session, r'^%s/[0-9a-f]{32}$' % support.DNFDAEMON_OBJECT_PATH)
        # close the session
        self.assertEqual(dbus.Boolean(True), self.iface.close_session(session))
        # closing non-existent session returns False
        self.assertEqual(dbus.Boolean(False),
                         self.iface.close_session(session))



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus
import os
import subprocess

import support


class RemoveTest(support.InstallrootCase):

    def setUp(self):
        super(RemoveTest, self).setUp()
        # install a package inside the installroot
        pkg_file = os.path.join(
            support.PROJECT_BINARY_DIR, "test/data/repos-rpm/rpm-repo1/one-1-1.noarch.rpm")
        res = subprocess.run(
            ["rpm", "--root", self.installroot, "-U", pkg_file])
        self.assertEqual(
            res.returncode, 0, "Installation of test package '{}' failed.".format(pkg_file))

    def test_remove_package(self):
        # remove an installed package
        self.iface_rpm.remove(['one'], dbus.Dictionary({}, signature='sv'))

        resolved, result = self.iface_goal.resolve(
            dbus.Dictionary({}, signature='sv'))
        self.sanitize_transaction(resolved)

        self.assertEqual(result, 0)
        self.assertCountEqual(
            resolved,
            dbus.Array([
                dbus.Struct((
                    dbus.String('Package'),         # object type
                    dbus.String('Remove'),          # action
                    dbus.String('User'),            # reason
                    dbus.Dictionary({               # transaction item attrs
                    }, signature=dbus.Signature('sv')),
                    dbus.Dictionary({               # package
                        dbus.String('full_nevra'): dbus.String('one-0:1-1.noarch', variant_level=1),
                        dbus.String('arch'): dbus.String('noarch', variant_level=1),
                        dbus.String('epoch'): dbus.String('0', variant_level=1),
                        dbus.String('evr'): dbus.String('1-1', variant_level=1),
                        dbus.String('name'): dbus.String('one', variant_level=1),
                        dbus.String('install_size'): dbus.UInt64(0, variant_level=1),
                        dbus.String('release'): dbus.String('1', variant_level=1),
                        dbus.String('repo_id'): dbus.String('@System', variant_level=1),
                        dbus.String('version'): dbus.String('1', variant_level=1),
                        dbus.String('from_repo_id'): dbus.String('<unknown>', variant_level=1),
                        dbus.String('reason'): dbus.String('External User', variant_level=1),
                    }, signature=dbus.Signature('sv'))),
                    signature=None)
            ], signature=dbus.Signature('(ua{sv})'))
        )

        self.iface_goal.do_transaction(dbus.Dictionary({}, signature='sv'))



repo_sack = base.get_repo_sack()

# Create repositories from system configuration files.
repo_sack.create_repos_from_system_configuration()

# Acquire a write lock on the system repository before loading it
base.lock_system_repo(libdnf5.utils.LockAccess_WRITE, libdnf5.utils.LockBlocking_BLOCKING)

# If out of date, downloads fresh repositories' metadata and loads the
# repositories into memory.
#
# load_repos() without any arguments says to load both the @System
# repository (the packages installed on the system, loaded from the rpmdb)
# and the available repositories.
repo_sack.load_repos()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus
import os

import support


class RepoTest(support.InstallrootCase):

    def test_list_repos(self):
        # get list of all repositories
        self.assertEqual(
            self.iface_repo.list({"repo_attrs": ["name", "enabled"]}),
            dbus.Array([
                dbus.Dictionary({
                    dbus.String('name'): dbus.String('Repository 1', variant_level=1),
                    dbus.String('enabled'): dbus.Boolean(True, variant_level=1),
                    dbus.String('id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
                dbus.Dictionary({
                    dbus.String('name'): dbus.String('Repository 2', variant_level=1),
                    dbus.String('enabled'): dbus.Boolean(True, variant_level=1),
                    dbus.String('id'): dbus.String('rpm-repo2', variant_level=1)},
                    signature=dbus.Signature('sv')),
            ],
                signature=dbus.Signature('a{sv}'))
        )

    def test_list_repos_spec(self):
        # get list of specified repositories
        self.assertEqual(
            self.iface_repo.list(
                {"repo_attrs": ["name", "enabled"], "patterns": ['rpm-repo1']}),
            dbus.Array([
                dbus.Dictionary({
                    dbus.String('name'): dbus.String('Repository 1', variant_level=1),
                    dbus.String('enabled'): dbus.Boolean(True, variant_level=1),
                    dbus.String('id'): dbus.String('rpm-repo1', variant_level=1)},
                    signature=dbus.Signature('sv')),
            ],
                signature=dbus.Signature('a{sv}'))
        )



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import os
import unittest

import libdnf5

import base_test_case


class TestRpmArch(unittest.TestCase):
    def test_get_supported_arches(self):
        arches = libdnf5.rpm.get_supported_arches()
        # at least one architecture was found
        self.assertTrue(len(arches) > 0)
        # the arches list is sorted and contains unique elements
        for i in range(len(arches) - 1):
            self.assertTrue(arches[i] < arches[i+1])

    def test_get_base_arch(self):
        self.assertEqual(libdnf5.rpm.get_base_arch("x86_64"), "x86_64")
        self.assertEqual(libdnf5.rpm.get_base_arch("amd64"), "x86_64")
        self.assertEqual(libdnf5.rpm.get_base_arch("UNKNOWN_ARCH"), "")



import libdnf5

# Create a new Base object
base = libdnf5.base.Base()

# Optionally load configuration from the config files.
base.load_config()

# For unit tests only - Prevent loading plugins from the host system.
# In production code, plugins are typically left enabled (default behavior).
base.get_config().get_plugins_option().set(False)

# Override the detected system architecture, similar to how the
# `--forcearch=aarch64` switch works in the dnf5 command line tool.
vars = base.get_vars().get()
vars.set("arch", "aarch64")

# This is sufficient for loading repositories and querying packages using the
# `aarch64` architecture.
# However, if you also want to run a transaction (e.g., you want to modify a
# foreign system from "outside" using `installroot`), you need to set the
# `ignorearch` option to instruct RPM to permit packages that are incompatible
# with the system architecture.
base.get_config().get_ignorearch_option().set(True)

# The base.setup() call configures the architecture for the solver, so the
# `arch` variable needs to be set beforehand.
base.setup()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.


import libdnf5

import base_test_case


class TestCompsIterators(base_test_case.BaseTestCase):
    def test_group_query_iterable(self):
        self.add_repo_repomd("repomd-comps-core")
        query = libdnf5.comps.GroupQuery(self.base)
        self.assertIsInstance(query.get_base().get(), libdnf5.base.Base)
        assert [grp.get_groupid() for grp in query]
        self.assertIsInstance(
            next(iter(query)).get_base().get(), libdnf5.base.Base)

    def test_environment_query_iterable(self):
        self.add_repo_repomd("repomd-comps-core-environment")
        query = libdnf5.comps.EnvironmentQuery(self.base)
        self.assertIsInstance(query.get_base().get(), libdnf5.base.Base)
        assert [env.get_environmentid() for env in query]
        self.assertIsInstance(
            next(iter(query)).get_base().get(), libdnf5.base.Base)



# Copyright Contributors to the DNF5 project.
# D-Bus XML documentation extension
#
# Copyright (C) 2021, Red Hat Inc.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Author: Marc-André Lureau <marcandre.lureau@redhat.com>
"""dbus-doc is a Sphinx extension that provides documentation from D-Bus XML."""

import os
import re
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    Iterator,
    List,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

import sphinx
from docutils import nodes
from docutils.nodes import Element, Node
from docutils.parsers.rst import Directive, directives
from docutils.parsers.rst.states import RSTState
from docutils.statemachine import StringList, ViewList
from sphinx.application import Sphinx
from sphinx.errors import ExtensionError
from sphinx.util import logging
from sphinx.util.docstrings import prepare_docstring
from sphinx.util.docutils import SphinxDirective, switch_source_input
from sphinx.util.nodes import nested_parse_with_titles

import dbusdomain
from dbusparser import parse_dbus_xml

logger = logging.getLogger(__name__)

__version__ = "1.0"


class DBusDoc:
    def __init__(self, sphinx_directive, dbusfile):
        self._cur_doc = None
        self._sphinx_directive = sphinx_directive
        self._dbusfile = dbusfile
        self._top_node = nodes.section()
        self.result = StringList()
        self.indent = ""

    def add_line(self, line: str, *lineno: int) -> None:
        """Append one line of generated reST to the output."""
        if line.strip():  # not a blank line
            self.result.append(self.indent + line, self._dbusfile, *lineno)
        else:
            self.result.append("", self._dbusfile, *lineno)

    def add_method(self, method):
        self.add_line(f".. dbus:method:: {method.name}")
        self.add_line("")
        self.indent += "   "
        for arg in method.in_args:
            self.add_line(f":arg {arg.signature} {arg.name}: {arg.doc_string}")
        for arg in method.out_args:
            self.add_line(f":ret {arg.signature} {arg.name}: {arg.doc_string}")
        self.add_line("")
        for line in prepare_docstring("\n" + method.doc_string):
            self.add_line(line)
        self.indent = self.indent[:-3]

    def add_signal(self, signal):
        self.add_line(f".. dbus:signal:: {signal.name}")
        self.add_line("")
        self.indent += "   "
        for arg in signal.args:
            self.add_line(f":arg {arg.signature} {arg.name}: {arg.doc_string}")
        self.add_line("")
        for line in prepare_docstring("\n" + signal.doc_string):
            self.add_line(line)
        self.indent = self.indent[:-3]

    def add_property(self, prop):
        self.add_line(f".. dbus:property:: {prop.name}")
        self.indent += "   "
        self.add_line(f":type: {prop.signature}")
        access = {"read": "readonly", "write": "writeonly", "readwrite": "readwrite"}[
            prop.access
        ]
        self.add_line(f":{access}:")
        if prop.emits_changed_signal:
            self.add_line(f":emits-changed: yes")
        self.add_line("")
        for line in prepare_docstring("\n" + prop.doc_string):
            self.add_line(line)
        self.indent = self.indent[:-3]

    def add_interface(self, iface):
        self.add_line(f".. dbus:interface:: {iface.name}")
        self.add_line("")
        self.indent += "   "
        for line in prepare_docstring("\n" + iface.doc_string):
            self.add_line(line)
        for method in iface.methods:
            self.add_method(method)
        for sig in iface.signals:
            self.add_signal(sig)
        for prop in iface.properties:
            self.add_property(prop)
        self.indent = self.indent[:-3]


def parse_generated_content(state: RSTState, content: StringList) -> List[Node]:
    """Parse a generated content by Documenter."""
    with switch_source_input(state, content):
        node = nodes.paragraph()
        node.document = state.document
        state.nested_parse(content, 0, node)

        return node.children


class DBusDocDirective(SphinxDirective):
    """Extract documentation from the specified D-Bus XML file"""

    has_content = True
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True

    def run(self):
        reporter = self.state.document.reporter

        try:
            source, lineno = reporter.get_source_and_line(
                self.lineno)  # type: ignore
        except AttributeError:
            source, lineno = (None, None)

        logger.debug("[dbusdoc] %s:%s: input:\n%s",
                     source, lineno, self.block_text)

        libdnf_docdir = os.path.abspath(".")
        qapidoc_srctree = os.path.join(libdnf_docdir, '..')
        dbusfile = qapidoc_srctree + "/" + self.arguments[0]
        with open(dbusfile, "rb") as f:
            xml_data = f.read()
        xml = parse_dbus_xml(xml_data)
        doc = DBusDoc(self, dbusfile)
        for iface in xml:
            doc.add_interface(iface)

        result = parse_generated_content(self.state, doc.result)
        return result


def setup(app: Sphinx) -> Dict[str, Any]:
    """Register dbus-doc directive with Sphinx"""
    app.add_config_value("dbusdoc_srctree", None, "env")
    app.add_directive("dbus-doc", DBusDocDirective)
    dbusdomain.setup(app)

    return dict(version=__version__, parallel_read_safe=True, parallel_write_safe=True)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5.base

import base_test_case


class TestGoal(base_test_case.BaseTestCase):
    def test_missing_setup_goal_resolve(self):
        # Create a new Base object
        base = libdnf5.base.Base()

        # Create a new empty Goal
        goal = libdnf5.base.Goal(base)

        # Try to resolve the goal without running base.setup()
        with self.assertRaisesRegex(libdnf5.exception.UserAssertionError,
                                    'libdnf5/base/goal.cpp:[0-9]+:.*libdnf5::Goal::resolve\\(\\):'
                                    ' API Assertion \'p_impl->base->is_initialized\\(\\)\' failed:'
                                    ' Base instance was not fully initialized by Base::setup\\(\\)'):
            goal.resolve()

    def test_missing_rpm_goal_resolve(self):
        # Try to resolve the goal using unknown RPM package

        goal = libdnf5.base.Goal(self.base)
        goal.add_install('unknown_pkg.rpm')

        with self.assertRaisesRegex(libdnf5.exception.RepoRpmError,
                                    'Failed to access RPM "unknown_pkg.rpm":'
                                    ' No such file or directory'):
            goal.resolve()

    def test_invalid_url_goal_resolve(self):
        # Try to resolve the goal using invalid URL

        goal = libdnf5.base.Goal(self.base)
        goal.add_install('https://i-dont-exist.com')

        with self.assertRaisesRegex(libdnf5.exception.RepoFileDownloadErrorNested, 'Failed to download files') as cm:
            goal.resolve()
        self.assertRegex(cm.exception.format(libdnf5.exception.FormatDetailLevel_WithDomainAndName),
                         'libdnf5::repo::FileDownloadError: Failed to download files\n'
                         ' libdnf5::repo::LibrepoError: Librepo error: Curl error \\([0-9]+\\):'
                         ' .* https://i-dont-exist.com/')
        with self.assertRaisesRegex(libdnf5.exception.Error, 'Librepo error: Curl error') as cm_nested:
            cm.exception.rethrow_if_nested()
        self.assertRegex(cm_nested.exception.format(libdnf5.exception.FormatDetailLevel_WithDomainAndName),
                         'libdnf5::Error: Librepo error: Curl error \\([0-9]+\\):'
                         ' .* https://i-dont-exist.com/')

    def test_unsupported_argument_add_remove(self):
        # Try passing unsupported argument to add_remove() method

        goal = libdnf5.base.Goal(self.base)

        with self.assertRaisesRegex(libdnf5.exception.RuntimeError, 'Unsupported argument for REMOVE action: pkg.rpm'):
            goal.add_remove('pkg.rpm')

    def test_group_rpm_reason_change_without_id(self):
        # Try changing group reason without group_id

        goal = libdnf5.base.Goal(self.base)

        with self.assertRaisesRegex(libdnf5.exception.UserAssertionError,
                                    'libdnf5/base/goal.cpp:[0-9]+: void libdnf5::Goal::add_rpm_reason_change\\(.*\\):'
                                    ' API Assertion \'reason != libdnf5::transaction::TransactionItemReason::GROUP || !group_id.empty\\(\\)\''
                                    ' failed: group_id is required for setting reason "GROUP"'):
            goal.add_rpm_reason_change(
                '@fake-group-spec', libdnf5.base.transaction.TransactionItemReason_GROUP, '')

    def test_log_events_wrapper(self):
        # Try accessing transaction.get_resolve_logs()
        spec = 'unknown_spec'

        goal = libdnf5.base.Goal(self.base)
        goal.add_install(spec)

        transaction = goal.resolve()
        logs = transaction.get_resolve_logs()
        self.assertTrue(logs)
        first_log = next(iter(logs))
        self.assertEqual(libdnf5.base.GoalProblem_NOT_FOUND,
                         first_log.get_problem())
        self.assertEqual(spec, first_log.get_spec())



# Create a group query.
groups = libdnf5.comps.GroupQuery(base)

# Filter the groups, the filters can be stacked one after another.
groups.filter_groupid("group-id")

# Iterate over the filtered groups in the query.
for group in groups:
    print(group.get_name())



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import dbus
import os
import shutil
import tempfile
import unittest

# the following settings come from cmake and are passed as ENVIRONMENT variables in CMakeLists.txt
PROJECT_BINARY_DIR = os.environ["PROJECT_BINARY_DIR"]

DNFDAEMON_BUS_NAME = 'org.rpm.dnf.v0'
DNFDAEMON_OBJECT_PATH = '/' + DNFDAEMON_BUS_NAME.replace('.', '/')

IFACE_SESSION_MANAGER = '{}.SessionManager'.format(DNFDAEMON_BUS_NAME)
IFACE_REPO = '{}.rpm.Repo'.format(DNFDAEMON_BUS_NAME)
IFACE_RPM = '{}.rpm.Rpm'.format(DNFDAEMON_BUS_NAME)
IFACE_GOAL = '{}.Goal'.format(DNFDAEMON_BUS_NAME)


class InstallrootCase(unittest.TestCase):

    def sanitize_transaction(self, resolved):
        '''Prepare resolved transaction for assert'''
        for object_type, action, reason, trans_item_attrs, pkg in resolved:
            # id of package depends on order of the repos in the sack which varies
            # between runs so we can't rely on the value
            pkg.pop('id')
            # also package size differs according to the builder
            pkg.pop('download_size')
            # TODO(mblaha): calculate correct replaces value
            if "replaces" in trans_item_attrs:
                trans_item_attrs.pop("replaces")

    def path_to_repo_rpm(self, repo, rpm):
        return os.path.join(self.repository_base, repo, rpm)

    def setUp(self):
        super(InstallrootCase, self).setUp()
        self.maxDiff = None

        self.installroot = tempfile.mkdtemp(prefix="dnf5daemon-test-")
        self.reposdir = os.path.join(
            PROJECT_BINARY_DIR, "test/data/repos-rpm-conf.d")
        self.config_file_path = os.path.join(
            self.installroot, 'etc/dnf/dnf.conf')
        self.repository_base = os.path.join(
            PROJECT_BINARY_DIR, "test/data/repos-rpm")
        os.makedirs(os.path.dirname(self.config_file_path), exist_ok=True)
        with open(self.config_file_path, 'w') as f:
            f.write('')

        self.bus = dbus.SystemBus()
        self.iface_session = dbus.Interface(
            self.bus.get_object(DNFDAEMON_BUS_NAME, DNFDAEMON_OBJECT_PATH),
            dbus_interface=IFACE_SESSION_MANAGER)
        # Prevent loading plugins from host by setting "plugins" to False
        self.session = self.iface_session.open_session({
            "config": {
                "config_file_path": self.config_file_path,
                "installroot": self.installroot,
                "plugins": "False",
                "cachedir": os.path.join(self.installroot, "var/cache/dnf"),
                "reposdir": self.reposdir,
            }
        })
        self.iface_repo = dbus.Interface(
            self.bus.get_object(DNFDAEMON_BUS_NAME, self.session),
            dbus_interface=IFACE_REPO)
        self.iface_rpm = dbus.Interface(
            self.bus.get_object(DNFDAEMON_BUS_NAME, self.session),
            dbus_interface=IFACE_RPM)
        self.iface_goal = dbus.Interface(
            self.bus.get_object(DNFDAEMON_BUS_NAME, self.session),
            dbus_interface=IFACE_GOAL)

    def tearDown(self):
        self.iface_session.close_session(self.session)
        shutil.rmtree(self.installroot)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import libdnf5

import base_test_case

import os
import shutil

FILE_LOGGER_FILENAME = "dnf5.log"


class TestFileLogger(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        config = self.base.get_config()
        config.logdir = os.path.join(
            config.installroot, "FileLoggerTestLogDir")
        self.full_log_path = os.path.join(config.logdir, FILE_LOGGER_FILENAME)

    def tearDown(self):
        super().tearDown()
        shutil.rmtree(os.path.dirname(self.full_log_path), ignore_errors=True)

    def test_file_logger_create_name(self):
        self.assertFalse(os.path.exists(self.full_log_path))
        _ = libdnf5.logger.create_file_logger(
            self.base, FILE_LOGGER_FILENAME)
        self.assertTrue(os.path.exists(self.full_log_path))

    def test_file_logger_add(self):
        log_router = self.base.get_logger()
        loggers_count_before = log_router.get_loggers_count()
        logger = libdnf5.logger.create_file_logger(self.base,
                                                   FILE_LOGGER_FILENAME)
        log_router.add_logger(logger)
        self.assertEqual(loggers_count_before + 1,
                         log_router.get_loggers_count())

    def test_with_global_logger(self):
        log_router = self.base.get_logger()
        global_logger = libdnf5.logger.GlobalLogger()
        global_logger.set(log_router.get(), libdnf5.logger.Logger.Level_DEBUG)
        logger = libdnf5.logger.create_file_logger(self.base,
                                                   FILE_LOGGER_FILENAME)
        log_router.add_logger(logger)

        # Run an action including librepo logging
        self.add_repo_repomd("repomd-repo1")

        with open(self.full_log_path) as logfile:
            self.assertTrue('[librepo]' in logfile.read())



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5


# Test using attributes instead of getters/setters
class TestStructAttributes(unittest.TestCase):
    def test_resolvespecsettings(self):
        settings = libdnf5.base.ResolveSpecSettings()
        settings.ignore_case = True
        settings.with_provides = False
        settings.with_filenames = False
        self.assertEqual(True, settings.ignore_case)
        self.assertEqual(False, settings.with_provides)
        self.assertEqual(False, settings.with_filenames)

    def test_goaljobsettings(self):
        settings = libdnf5.base.GoalJobSettings()
        settings.best = libdnf5.base.GoalSetting_SET_TRUE
        settings.skip_unavailable = False
        settings.to_repo_ids = ('repo1', 'repo2')
        self.assertEqual(libdnf5.base.GoalSetting_SET_TRUE, settings.best)
        self.assertEqual(False, settings.skip_unavailable)
        self.assertEqual(('repo1', 'repo2'), settings.to_repo_ids)

    def test_rpmchangelog(self):
        changelog = libdnf5.rpm.Changelog(1234, 'Jo Anne', 'Version bump')
        self.assertEqual(1234, changelog.timestamp)
        self.assertEqual('Jo Anne', changelog.author)
        self.assertEqual('Version bump', changelog.text)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.


import os
import shutil
import tempfile
import unittest

import libdnf5


PROJECT_BINARY_DIR = os.environ["PROJECT_BINARY_DIR"]
PROJECT_SOURCE_DIR = os.environ["PROJECT_SOURCE_DIR"]


class BaseTestCase(unittest.TestCase):
    def setUp(self):
        self.base = libdnf5.base.Base()

        self.temp_dir = tempfile.mkdtemp(prefix="libdnf5_python3_unittest.")

        config = self.base.get_config()
        config.installroot = os.path.join(self.temp_dir, "installroot")
        config.cachedir = os.path.join(self.temp_dir, "cache")
        config.optional_metadata_types = libdnf5.conf.OPTIONAL_METADATA_TYPES

        # Prevent loading plugins from the host
        config.plugins = False

        vars = self.base.get_vars().get()
        vars.set("arch", "x86_64")

        self.base.setup()

        self.repo_sack = self.base.get_repo_sack()
        self.package_sack = self.base.get_rpm_package_sack()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def _add_repo(self, repoid, repo_path, load=True):
        """
        Add a repo from `repo_path`.
        """
        repo = self.repo_sack.create_repo(repoid)
        repo.get_config().baseurl = "file://" + repo_path

        if load:
            self.repo_sack.load_repos(libdnf5.repo.Repo.Type_AVAILABLE)

        return repo

    def add_repo_repomd(self, repoid, load=True):
        """
        Add a repo from PROJECT_SOURCE_DIR/test/data/repos-repomd/<repoid>/repodata
        """
        repo_path = os.path.join(
            PROJECT_SOURCE_DIR, "test/data/repos-repomd", repoid)
        return self._add_repo(repoid, repo_path, load)

    def add_repo_rpm(self, repoid, load=True):
        """
        Add a repo from PROJECT_BINARY_DIR/test/data/repos-rpm/<repoid>/repodata
        """
        repo_path = os.path.join(
            PROJECT_BINARY_DIR, "test/data/repos-rpm", repoid)
        return self._add_repo(repoid, repo_path, load)

    def add_repo_solv(self, repoid):
        """
        Add a repo from PROJECT_SOURCE_DIR/test/data/repos-solv/<repoid>.repo
        """
        repo_path = os.path.join(
            PROJECT_SOURCE_DIR, "test/data/repos-solv", repoid + ".repo")
        return self.repo_sack.create_repo_from_libsolv_testcase(repoid, repo_path)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import libdnf5.base

import base_test_case


class TestTransaction(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.add_repo_repomd("repomd-repo1")

    def test_check_gpg_signatures_no_gpgcheck(self):
        goal = libdnf5.base.Goal(self.base)
        goal.add_rpm_install("pkg")
        transaction = goal.resolve()

        self.assertEqual(1, transaction.get_transaction_packages_count())
        self.assertTrue(transaction.check_gpg_signatures())
        self.assertEqual(('Warning: skipped OpenPGP checks for 1 package from repository: repomd-repo1',),
                         transaction.get_gpg_signature_problems())

    def test_check_gpg_signatures_fail(self):
        # Both options need to be available
        self.base.get_config().gpgcheck = True
        self.base.get_config().pkg_gpgcheck = True

        goal = libdnf5.base.Goal(self.base)
        goal.add_rpm_install("pkg")
        transaction = goal.resolve()

        self.assertEqual(1, transaction.get_transaction_packages_count())
        self.assertFalse(transaction.check_gpg_signatures())
        self.assertTrue(len(transaction.get_gpg_signature_problems()) > 0)



# Create a goal, which is a class that allows to add items for resolving into
# a transaction.
goal = libdnf5.base.Goal(base)

# Add an RPM package named "one" for installation into the goal.
goal.add_rpm_install("one")

# Resolve the goal, create a transaction object.
#
# The argument is `allow_erasing`, a flag indicating whether to allow removing
# packages in the resolved transaction.
transaction = goal.resolve()

# We can iterate over the resolved transaction and inspect the packages.
print("Resolved transaction:")
for tspkg in transaction.get_transaction_packages():
    print(tspkg.get_package().get_nevra(), ": ",
          libdnf5.base.transaction.transaction_item_action_to_string(tspkg.get_action()))

# This class demonstrates user-defined callbacks for the package downloads.
#
# The callbacks are implemented by inheriting from the callbacks base class
# and overriding its methods.
#
# We only override one of the callbacks here, see
# `libdnf5.repo.DownloadCallbacks` documentation for a complete list.


class PackageDownloadCallbacks(libdnf5.repo.DownloadCallbacks):
    def mirror_failure(self, user_cb_data, msg, url=""):
        print("Mirror failure: ", msg)
        return 0


downloader_callbacks = PackageDownloadCallbacks()
base.set_download_callbacks(
    libdnf5.repo.DownloadCallbacksUniquePtr(downloader_callbacks))

# Download the packages.
transaction.download()

# A class for defining the RPM transaction callbacks.
#
# Again, only a callback for when an RPM package installation starts, for a
# complete list of the callbacks see `libdnf5.rpm.TransactionCallbacks`
# documentation.


class TransactionCallbacks(libdnf5.rpm.TransactionCallbacks):
    def install_start(self, item, total=0):
        print(libdnf5.base.transaction.transaction_item_action_to_string(item.get_action()), " ",
              item.get_package().get_nevra())


transaction_callbacks = TransactionCallbacks()
transaction_callbacks_ptr = libdnf5.rpm.TransactionCallbacksUniquePtr(
    transaction_callbacks)
transaction.set_callbacks(transaction_callbacks_ptr)

# Add transaction metadata to be stored in the history database.
transaction.set_description("install package one")

# Run the transaction.
print("Running the transaction:")
transaction.run()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5.utils


class TestIsXPattern(unittest.TestCase):
    def test_is_file_pattern(self):
        self.assertFalse(libdnf5.utils.is_file_pattern(''))
        self.assertFalse(libdnf5.utils.is_file_pattern('no_file_pattern'))
        self.assertFalse(libdnf5.utils.is_file_pattern('no_file/pattern'))
        self.assertTrue(libdnf5.utils.is_file_pattern('/pattern'))
        self.assertTrue(libdnf5.utils.is_file_pattern('*/pattern'))

    def test_is_glob_pattern(self):
        self.assertFalse(libdnf5.utils.is_glob_pattern(''))
        self.assertFalse(libdnf5.utils.is_glob_pattern('no_glob_pattern'))
        self.assertTrue(libdnf5.utils.is_glob_pattern('glob*_pattern'))
        self.assertTrue(libdnf5.utils.is_glob_pattern('glob[sdf]_pattern'))
        self.assertTrue(libdnf5.utils.is_glob_pattern('glob?_pattern'))



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import os
import pathlib
import tempfile

import base_test_case

PROJECT_BINARY_DIR = os.environ["PROJECT_BINARY_DIR"]
PROJECT_SOURCE_DIR = os.environ["PROJECT_SOURCE_DIR"]


class TestTutorial(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.installroot = self.base.get_config().installroot
        self.cachedir = self.base.get_config().cachedir
        self.baseurl = pathlib.Path(os.path.join(
            PROJECT_BINARY_DIR, "test/data/repos-rpm/rpm-repo1/")).as_uri()

    def test_create_base(self):
        file = ""
        with open("tutorial/session/create_base.py", "r") as f:
            file += f.read()

        exec(file, {'installroot': self.installroot, 'cachedir': self.cachedir})

    def test_load_repo(self):
        file = ""
        with open("tutorial/session/create_base.py", "r") as f:
            file += f.read()

        with open("tutorial/repo/load_repo.py", "r") as f:
            file += f.read()

        exec(file, {'installroot': self.installroot,
                    'cachedir': self.cachedir, 'baseurl': self.baseurl})

    def test_load_system_repo(self):
        # TODO(nsella) This example does not 'compile' yet
        # file = ""
        # with open("tutorial/session/create_base.py", "r") as f:
        #     file += f.read()

        # with open("tutorial/repo/load_system_repos.py", "r") as f:
        #     file += f.read()

        # exec(file, { 'installroot': self.installroot, 'baseurl': self.baseurl })
        pass

    def test_query(self):
        file = ""
        with open("tutorial/session/create_base.py", "r") as f:
            file += f.read()

        with open("tutorial/repo/load_repo.py", "r") as f:
            file += f.read()

        with open("tutorial/query/query.py", "r") as f:
            file += f.read()

        exec(file, {'installroot': self.installroot,
                    'cachedir': self.cachedir, 'baseurl': self.baseurl})

    def test_transaction(self):
        file = ""
        with open("tutorial/session/create_base.py", "r") as f:
            file += f.read()

        with open("tutorial/repo/load_repo.py", "r") as f:
            file += f.read()

        with open("tutorial/transaction/transaction.py", "r") as f:
            file += f.read()

        tmp_path = os.getcwd()
        exec(file, {'installroot': self.installroot,
                    'cachedir': self.cachedir, 'baseurl': self.baseurl})
        os.chdir(tmp_path)

    def test_force_arch(self):
        file = ""
        with open("tutorial/session/force_arch.py", "r") as f:
            file += f.read()

        exec(file, {'installroot': self.installroot, 'cachedir': self.cachedir})



# Copyright Contributors to the DNF5 project.
# D-Bus XML documentation extension, compatibility gunk for <sphinx4
#
# Copyright (C) 2021, Red Hat Inc.
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Author: Marc-André Lureau <marcandre.lureau@redhat.com>
"""dbus-doc is a Sphinx extension that provides documentation from D-Bus XML."""

from sphinx.application import Sphinx
from sphinx.util.docutils import SphinxDirective
from typing import Any, Dict


class FakeDBusDocDirective(SphinxDirective):
    has_content = True
    required_arguments = 1

    def run(self):
        return []


def setup(app: Sphinx) -> Dict[str, Any]:
    """Register a fake dbus-doc directive with Sphinx"""
    app.add_directive("dbus-doc", FakeDBusDocDirective)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5.base

import base_test_case


class TestRepo(base_test_case.BaseTestCase):
    class DownloadCallbacks(libdnf5.repo.DownloadCallbacks):

        def __init__(self):
            super().__init__()
            self.last_user_data = None
            self.start_cnt = 0
            self.end_cnt = 0
            self.end_error_message = None
            self.fastest_mirror_cnt = 0
            self.handle_mirror_failure_cnt = 0

        def add_new_download(self, user_data, description, total_to_download):
            self.start_cnt += 1
            self.last_user_data = user_data
            return 0

        def end(self, user_cb_data, status, error_message):
            self.end_cnt += 1
            self.end_error_message = error_message
            return 0

        def fastest_mirror(self, user_cb_data, stage, ptr):
            self.fastest_mirror_cnt += 1

        def handle_mirror_failure(self, user_cb_data, msg, url, metadata):
            self.handle_mirror_failure_cnt += 1
            return 0

    class RepoCallbacks(libdnf5.repo.RepoCallbacks):
        repokey_import_cnt = 0

        def repokey_import(self, id, user_id, fingerprint, url, timestamp):
            self.repokey_import_cnt += 1
            return True

    def test_load_repo(self):
        repoid = "repomd-repo1"
        repo = self.add_repo_repomd(repoid, load=False)

        USER_DATA = 25
        repo.set_user_data(USER_DATA)
        self.assertEqual(repo.get_user_data(), USER_DATA)

        dl_cbs = self.DownloadCallbacks()
        self.base.set_download_callbacks(
            libdnf5.repo.DownloadCallbacksUniquePtr(dl_cbs))

        cbs = self.RepoCallbacks()
        # TODO(lukash) try to wrap the creation of the unique_ptr so that cbs
        # can be passed directly to repo.set_callbacks
        repo.set_callbacks(libdnf5.repo.RepoCallbacksUniquePtr(cbs))

        self.repo_sack.load_repos(libdnf5.repo.Repo.Type_AVAILABLE)

        self.assertEqual(dl_cbs.last_user_data, USER_DATA)

        self.assertEqual(dl_cbs.start_cnt, 1)
        self.assertEqual(dl_cbs.end_cnt, 1)
        self.assertEqual(dl_cbs.end_error_message, None)

        self.assertEqual(dl_cbs.fastest_mirror_cnt, 0)
        self.assertEqual(dl_cbs.handle_mirror_failure_cnt, 0)
        self.assertEqual(cbs.repokey_import_cnt, 0)

    def test_load_repo_overload(self):
        repoid = "repomd-repo1"
        repo = self.add_repo_repomd(repoid, load=False)

        dl_cbs = self.DownloadCallbacks()
        self.base.set_download_callbacks(
            libdnf5.repo.DownloadCallbacksUniquePtr(dl_cbs))

        cbs = self.RepoCallbacks()
        repo.set_callbacks(libdnf5.repo.RepoCallbacksUniquePtr(cbs))

        self.repo_sack.load_repos()

        self.assertEqual(dl_cbs.end_cnt, 1)
        self.assertEqual(dl_cbs.end_error_message, None)

        self.assertEqual(dl_cbs.fastest_mirror_cnt, 0)
        self.assertEqual(dl_cbs.handle_mirror_failure_cnt, 0)
        self.assertEqual(cbs.repokey_import_cnt, 0)

    def test_iterate_config_options(self):
        repoid = "repomd-repo1"
        repo = self.add_repo_repomd(repoid, load=False)
        config = repo.get_config()

        config.proxy = 'abcd'

        proxy_option = None
        for option in config.opt_binds():
            if option.first == 'proxy':
                proxy_option = (option.first, option.second.get_value_string())

        self.assertEqual(proxy_option, ('proxy', 'abcd'))



# Copyright Contributors to the DNF5 project.
# Copyright (C) 2021 Red Hat, Inc.
# SPDX-License-Identifier: LGPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.


import libdnf5

import base_test_case


class TestPackageQuerySetOperators(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.add_repo_solv("solv-24pkgs")

    def test_update(self):
        # packages with releases: 1, 2
        q1 = libdnf5.rpm.PackageQuery(self.base)
        q1.filter_release(["1", "2"])
        self.assertEqual(q1.size(), 2)

        # packages with releases: 2, 3
        q2 = libdnf5.rpm.PackageQuery(self.base)
        q2.filter_release(["2", "3"])
        self.assertEqual(q2.size(), 2)

        # union is all 3 packages
        q1.update(q2)
        self.assertEqual(q1.size(), 3)

    def test_difference(self):
        # packages with releases: 1, 2
        q1 = libdnf5.rpm.PackageQuery(self.base)
        q1.filter_release(["1", "2"])
        self.assertEqual(q1.size(), 2)

        q2 = libdnf5.rpm.PackageQuery(self.base)
        q2.filter_release(["2", "3"])
        self.assertEqual(q2.size(), 2)

        q1.difference(q2)
        self.assertEqual(q1.size(), 1)

        # difference is the package with release 1
        pkg = list(q1)[0]
        self.assertEqual(pkg.get_release(), "1")

    def test_intersection(self):
        # packages with releases: 1, 2
        q1 = libdnf5.rpm.PackageQuery(self.base)
        q1.filter_release(["1", "2"])
        self.assertEqual(q1.size(), 2)

        # packages with releases: 2, 3
        q2 = libdnf5.rpm.PackageQuery(self.base)
        q2.filter_release(["2", "3"])
        self.assertEqual(q2.size(), 2)

        # intersection is the package with release 2
        q1.intersection(q2)
        self.assertEqual(q1.size(), 1)

        pkg = list(q1)[0]
        self.assertEqual(pkg.get_release(), "2")



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5

import base_test_case


class TestReldepList(base_test_case.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.add_repo_repomd("repomd-repo1")

    def test_add(self):
        a = libdnf5.rpm.Reldep(self.base, "python3-labirinto = 4.2.0")
        b = libdnf5.rpm.Reldep(self.base, "(lab-list if labirinto.txt)")

        list1 = libdnf5.rpm.ReldepList(self.base)
        self.assertEqual(list1.size(), 0)
        list1.add(a)
        # TODO(jrohel): Shall we have add() with ReldepId as a part of public API?
        list1.add(b.get_id())
        list1.add_reldep("delgado > 1.2")
        self.assertEqual(list1.size(), 3)

    def test_get(self):
        a = libdnf5.rpm.Reldep(self.base, "python3-labirinto = 4.2.0")
        b = libdnf5.rpm.Reldep(self.base, "(lab-list if labirinto.txt)")
        c = libdnf5.rpm.Reldep(self.base, "(labirinto unless labirinto_c)")

        list1 = libdnf5.rpm.ReldepList(self.base)
        list1.add(a)
        list1.add(b)
        list1.add(c)

        self.assertEqual(list1.get(0), a)
        self.assertEqual(list1.get(1).to_string(),
                         "(lab-list if labirinto.txt)")
        self.assertEqual(list1.get_id(2).id, c.get_id().id)

    def test_compare(self):
        a = libdnf5.rpm.Reldep(self.base, "python3-labirinto = 4.2.0")
        b = libdnf5.rpm.Reldep(self.base, "(lab-list if labirinto.txt)")

        list1 = libdnf5.rpm.ReldepList(self.base)
        list2 = libdnf5.rpm.ReldepList(self.base)
        self.assertEqual(list1, list2)

        list1.add(a)
        self.assertNotEqual(list1, list2)

        list2.add(a)
        list1.add(b)
        list2.add(b)
        self.assertEqual(list1, list2)

    def test_append(self):
        a = libdnf5.rpm.Reldep(self.base, "python3-labirinto = 4.2.0")
        b = libdnf5.rpm.Reldep(self.base, "(lab-list if labirinto.txt)")
        c = libdnf5.rpm.Reldep(self.base, "(labirinto unless labirinto_c)")
        d = libdnf5.rpm.Reldep(self.base, "labirinto.txt")

        list1 = libdnf5.rpm.ReldepList(self.base)
        list1.add(a)
        list1.add(b)
        list1.add_reldep("delgado > 1.2")

        list2 = libdnf5.rpm.ReldepList(self.base)
        list2.add(c)
        list2.add(d)

        list1.append(list2)
        self.assertEqual(list1.size(), 5)
        self.assertEqual(list1.get(0), a)
        self.assertEqual(list1.get(1), b)
        self.assertEqual(list1.get(2).to_string(), "delgado > 1.2")
        self.assertEqual(list1.get(3), c)
        self.assertEqual(list1.get(4), d)

    def test_iterator(self):
        a = libdnf5.rpm.Reldep(self.base, "python3-labirinto = 4.2.0")
        b = libdnf5.rpm.Reldep(self.base, "(lab-list if labirinto.txt)")
        c = libdnf5.rpm.Reldep(self.base, "(labirinto unless labirinto_c)")
        d = libdnf5.rpm.Reldep(self.base, "labirinto.txt")
        expect = [a, b, c, d]

        list1 = libdnf5.rpm.ReldepList(self.base)
        for reldep in expect:
            list1.add(reldep)

        # check if begin() points to the first Reldep
        it1 = list1.begin()
        self.assertEqual(it1.value(), a)

        # test next()
        it1.next()
        self.assertEqual(it1.value(), b)

        # check Python style iterator
        pit = iter(list1)
        self.assertEqual(next(pit), a)
        self.assertEqual(next(pit), b)
        self.assertEqual(next(pit, None), c)
        self.assertEqual(next(pit, None), d)
        self.assertEqual(next(pit, None), None)

        # test iteration using a for loop
        result = []
        for reldep in list1:
            result.append(reldep)
        self.assertEqual(result, expect)

    # add_reldep_with_glob uses libsolvs Dataiterator which needs the actual packages
    def test_add_reldep_with_glob(self):
        list1 = libdnf5.rpm.ReldepList(self.base)
        list1.add_reldep_with_glob("pkg*")

        expected = ["pkg", "pkg.conf", "pkg.conf.d", "pkg-libs",
                    "pkg", "pkg", "pkg", "pkg", "pkg", "pkg"]
        # TODO(dmach): implement __str__()
        result = [reldep.to_string() for reldep in list1]
        self.assertEqual(expected, result)



class PackageDownloadCallbacks(libdnf5.repo.DownloadCallbacks):
    # List of descriptions of downloads.
    download_descriptions = []

    def add_new_download(self, user_data, description, total_to_download):
        print(f"Started downloading: {description}, size: {total_to_download}")

        # Store the message describing the new download and return its index in the list.
        self.download_descriptions.append(description)
        return len(self.download_descriptions) - 1

    def end(self, user_cb_data, status, msg):
        # Get the description based on the index passed in user_cb_data.
        description = self.download_descriptions[user_cb_data]

        if status is libdnf5.repo.DownloadCallbacks.TransferStatus_SUCCESSFUL:
            print(f"Downloaded: {description}")
        elif status is libdnf5.repo.DownloadCallbacks.TransferStatus_ALREADYEXISTS:
            print(f"Skipped to download: {description}: {msg}")
        else:
            print(f"Failed to download: {description}: {msg}")

        return 0


# Set the download callbacks.
downloader_callbacks = PackageDownloadCallbacks()
base.set_download_callbacks(
    libdnf5.repo.DownloadCallbacksUniquePtr(downloader_callbacks))

# Download the packages.
transaction.download()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 2.1 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import libdnf5


class Plugin(libdnf5.plugin.IPlugin):
    def __init__(self, data):
        super(Plugin, self).__init__(data)
        self.base = self.get_base()

    @staticmethod
    def get_api_version():
        return libdnf5.PluginAPIVersion(2, 1)

    @staticmethod
    def get_name():
        return 'plugin'

    @staticmethod
    def get_version():
        return libdnf5.plugin.Version(0, 1, 0)

    def get_attribute(self, name):
        attributes = {'author_name': 'Jaroslav Rohel',
                      'author_email': 'jrohel@redhat.com',
                      'description': 'Libdnf5 plugin written in Python. Processes the LOAD_CONFIG_FROM_FILE hook '
                                     'and writes the current value of the "skip_if_unavailable" option to the log.'}
        return attributes.get(name, None)

    def repos_configured(self):
        logger = self.base.get_logger()
        config = self.base.get_config()
        logger.info(self.get_name() + ' - skip_if_unavailable = ' +
                    str(config.skip_if_unavailable))
        print(self.get_name() + ' - skip_if_unavailable = ' +
              str(config.skip_if_unavailable))
        return True

    def finish(self):
        pass



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import unittest

import libdnf5.base


class TestBase(unittest.TestCase):
    def test_base(self):
        base = libdnf5.base.Base()
        loger = base.get_logger()
        config = base.get_config()
        repo_sack = base.get_repo_sack()
        package_sack = base.get_rpm_package_sack()

    def test_weak_ptr(self):
        # Creates a new Base object
        base = libdnf5.base.Base()

        # Gets a WeakPtr pointing to Vars in the Base object
        vars = base.get_vars()

        # Creates a copy of WeakPtr
        vars2 = vars

        # Base is valid -> WeakPtr is valid. Sets "test_variable" using WeakPtr vars.
        vars.set("test_variable", "value1")

        # Base is valid -> WeakPtr is valid. Gets value of "test_variable" using copy of WeakPtr vars2.
        self.assertEqual(vars2.get_value("test_variable"), "value1")

        # Invalidates Base object
        base = None

        # Base object is invalid. -> Both WeakPtr are invalid. The code must throw an exception.
        # Raises an AssertionError that is not caught by the SWIG binding.
        # with self.assertRaisesRegex(libdnf5.exception.AssertionError, 'Dereferencing an invalidated WeakPtr'):
        #    vars.get_value("test_variable")
        # with self.assertRaisesRegex(libdnf5.exception.AssertionError, 'Dereferencing an invalidated WeakPtr'):
        #    vars2.get_value("test_variable")

    def test_non_existing_config_load(self):
        # Try to load configuration from non-existing path
        base = libdnf5.base.Base()
        base.get_config().config_file_path = 'this-path-does-not-exist.conf'

        # Checking the exception.
        with self.assertRaisesRegex(libdnf5.exception.MissingConfigErrorNested,
                                    'Configuration file "this-path-does-not-exist.conf" not found') as cm:
            base.load_config()
        self.assertRegex(cm.exception.format(libdnf5.exception.FormatDetailLevel_WithDomainAndName),
                         '^libdnf5::MissingConfigError: Configuration file "this-path-does-not-exist.conf" not found\n'
                         ' libdnf5::utils::fs::FileSystemError: cannot open file')

        # Checking the nested exception.
        with self.assertRaisesRegex(libdnf5.exception.FileSystemError,
                                    'cannot open file') as cm_nested:
            cm.exception.rethrow_if_nested()
        self.assertRegex(cm_nested.exception.format(libdnf5.exception.FormatDetailLevel_WithDomainAndName),
                         '^libdnf5::utils::fs::FileSystemError: cannot open file')



import libdnf5

# Create a new Base object
base = libdnf5.base.Base()

# Optionally set installroot.
#
# Installroot is set to '/' when we're working with the system, but we can set
# it to a different location. The Base instance will then work with the
# installroot directory tree as its root for the rest of its lifetime.
base_config = base.get_config()
base_config.installroot = installroot

# Optionally prevent of loading of plugins. Plugins are loaded by default from the host
base_config.plugins = False

# Optionally load configuration from the config files.
#
# The Base's config is initialized with default values, one of which is
# "config_file_path". This contains the default path to the config file
# ("/etc/dnf/dnf.conf"). If the file does not exist the distribution config file
# is loaded. Function also loads configuration files from distribution and
# user ("/etc/dnf/libdnf5.conf.d") drop-in directories.
# Optionally set a custom value to "config_file_path" before calling this method
# to load configuration from a another configuration file.
base.load_config()

# Optionally you can set and get vars
# vars = base.get_vars().get()
# vars.set('releasever', '33')
#
# Its value can be printed by get_value method
# print(vars.get_value('releasever'))
#
# There are plans in the future to support the methods get() or get_priority()

# Optionally set cachedir.
#
# By default, system cachedir or user cachedir is used, but we can set it to a
# different location. This is useful for tests.
base_config.cachedir = cachedir

# Load vars and do other initialization (of libsolv pool, etc.) based on the
# configuration.  Locks the installroot and varsdir configuration values so
# that they can't be changed.
base.setup()



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import os
import unittest

import libdnf5.base

import base_test_case


class TestFileDownloader(base_test_case.BaseTestCase):
    USER_DATA = 25

    def test_package_downloader(self):
        class DownloadCallbacks(libdnf5.repo.DownloadCallbacks):
            USER_CB_DATA = 5
            test_instance = None

            def __init__(self):
                super(DownloadCallbacks, self).__init__()
                self.start_cnt = 0
                self.progress_cnt = 0
                self.mirror_failure_cnt = 0
                self.end_cnt = 0
                self.end_status = None
                self.end_msg = None

            def add_new_download(self, user_data, description, total_to_download):
                self.start_cnt += 1
                self.test_instance.assertEqual(
                    user_data, self.test_instance.USER_DATA)
                return self.USER_CB_DATA

            def end(self, user_cb_data, status, msg):
                self.end_cnt += 1
                self.test_instance.assertEqual(user_cb_data, self.USER_CB_DATA)
                self.end_status = status
                self.end_msg = msg
                return 0

            def progress(self, user_cb_data, total_to_download, downloaded):
                self.progress_cnt += 1
                self.test_instance.assertEqual(user_cb_data, self.USER_CB_DATA)
                return 0

            def mirror_failure(self, user_cb_data, msg, url):
                self.mirror_failure_cnt += 1
                self.test_instance.assertEqual(user_cb_data, self.USER_CB_DATA)
                return 0

        DownloadCallbacks.test_instance = self

        source_file_path = os.path.join(
            base_test_case.PROJECT_SOURCE_DIR, "test/data/keys/key.pub")
        source_url = "file://" + source_file_path
        dest_file_path = os.path.join(self.temp_dir, "file_downloader.pub")

        cbs = DownloadCallbacks()
        self.base.set_download_callbacks(
            libdnf5.repo.DownloadCallbacksUniquePtr(cbs))

        file_downloader = libdnf5.repo.FileDownloader(self.base)
        file_downloader.add(source_url, dest_file_path, self.USER_DATA)
        file_downloader.download()

        self.assertEqual(cbs.start_cnt, 1)
        self.assertGreaterEqual(cbs.progress_cnt, 1)
        self.assertEqual(cbs.mirror_failure_cnt, 0)
        self.assertEqual(cbs.end_cnt, 1)

        self.assertEqual(
            cbs.end_status, DownloadCallbacks.TransferStatus_SUCCESSFUL)
        self.assertEqual(cbs.end_msg, None)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#!/usr/bin/python3

#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

"""
This is an example how to perform a system-upgrade with dnf5daemon-server.
"""

import dbus

DNFDAEMON_BUS_NAME = 'org.rpm.dnf.v0'
DNFDAEMON_OBJECT_PATH = '/' + DNFDAEMON_BUS_NAME.replace('.', '/')

IFACE_SESSION_MANAGER = '{}.SessionManager'.format(DNFDAEMON_BUS_NAME)
IFACE_RPM = '{}.rpm.Rpm'.format(DNFDAEMON_BUS_NAME)
IFACE_GOAL = '{}.Goal'.format(DNFDAEMON_BUS_NAME)


bus = dbus.SystemBus()
iface_session = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, DNFDAEMON_OBJECT_PATH),
    dbus_interface=IFACE_SESSION_MANAGER)

# set the releasever to the new distribution release
session = iface_session.open_session(
    dbus.Dictionary({"releasever": "40"}, signature=dbus.Signature('sv')))

iface_rpm = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, session),
    dbus_interface=IFACE_RPM)
iface_goal = dbus.Interface(
    bus.get_object(DNFDAEMON_BUS_NAME, session),
    dbus_interface=IFACE_GOAL)

# Add system upgrade to the transaction
options = {
    "mode": "distrosync",
}
iface_rpm.system_upgrade(options)

# resolve the transaction
resolved, result = iface_goal.resolve({})

# now you can print the transaction table and ask the user for confirmation
print("Resolved.")

if result == 0:
    # execute the transaction offline (durint the next reboot)
    iface_goal.do_transaction({"offline": True}, timeout=2000)
    print("Reboot to continue with system upgrade...")
else:
    errors = iface_goal.get_transaction_problems_string()
    print("Errors while resolving the transaction:")
    for error in errors:
        print(error)



# Copyright Contributors to the DNF5 project.
# Copyright Contributors to the libdnf project.
# SPDX-License-Identifier: GPL-2.0-or-later
#
# This file is part of libdnf: https://github.com/rpm-software-management/libdnf/
#
# Libdnf is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Libdnf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libdnf.  If not, see <https://www.gnu.org/licenses/>.

import io
import unittest

import libdnf5.logger


# Tests overloading of log() method
class libdnf5LoggerCB1(libdnf5.logger.Logger):
    def __init__(self, stream):
        super(libdnf5LoggerCB1, self).__init__()
        self._stream = stream

    def log_line(self, level, message):
        self._stream.write("{}: {}\n".format(
            self.level_to_cstr(level), message))

    def write(self, time, pid, level, message):
        self._stream.write("Bad message\n")


# Tests overloading of write() method
class libdnf5LoggerCB2(libdnf5.logger.Logger):
    def __init__(self, stream):
        super(libdnf5LoggerCB2, self).__init__()
        self._stream = stream

    def write(self, time, pid, level, message):
        self._stream.write("{}: {}\n".format(
            self.level_to_cstr(level), message))


class TestLoggers(unittest.TestCase):
    def test_logger(self):
        expected_logger_content = "CRITICAL: Critical message\n" \
                                  "ERROR: Error message\n"       \
                                  "WARNING: Warning message\n"   \
                                  "NOTICE: Notice message\n"     \
                                  "INFO: Info message\n"         \
                                  "DEBUG: Debug message\n"       \
                                  "TRACE: Trace message\n"

        stream1 = io.StringIO()
        logger1 = libdnf5LoggerCB1(stream1)
        stream2 = io.StringIO()
        logger2 = libdnf5LoggerCB2(stream2)
        for logger in logger1, logger2:
            logger.critical("Critical message")
            logger.error("Error message")
            logger.warning("Warning message")
            logger.notice("Notice message")
            logger.info("Info message")
            logger.debug("Debug message")
            logger.trace("Trace message")
        content1 = stream1.getvalue()
        self.assertEqual(content1, expected_logger_content)
        content2 = stream2.getvalue()
        self.assertEqual(content2, expected_logger_content)

    def test_logrouter(self):
        expected_logger_content = "INFO: Info message\n"         \
                                  "DEBUG: Debug message\n"       \
                                  "TRACE: Trace message\n"       \
                                  "CRITICAL: Critical message\n" \
                                  "ERROR: Error message\n"       \
                                  "WARNING: Warning message\n"   \
                                  "NOTICE: Notice message\n"     \
                                  "INFO: Info message\n"         \
                                  "DEBUG: Debug message\n"       \
                                  "TRACE: Trace message\n"       \
                                  "INFO: Info additional message\n"

        # ====================
        #  1. Create a LogRouter instance with one MemoryBufferLogger instances attached.
        #  ====================
        # Create log router.
        log_router = libdnf5.logger.LogRouter()
        # Create circular memory buffer logger with capacity 10 messages (4 pre-allocated from start).
        max_items_to_keep = 10
        reserve = 4
        memory_buffer_logger = libdnf5.logger.MemoryBufferLogger(
            max_items_to_keep, reserve)
        logger_uniq_ptr = libdnf5.logger.LoggerUniquePtr(memory_buffer_logger)
        log_router.add_logger(logger_uniq_ptr)

        # Test the number of registered loggers.
        self.assertEqual(log_router.get_loggers_count(), 1)

        # ====================
        # 2. Write messages into log_router. They will be routed into memory_buffer_logger.
        # ====================
        for i in range(2):
            log_router.log(libdnf5.logger.Logger.Level_CRITICAL,
                           "Critical message")
            log_router.log(libdnf5.logger.Logger.Level_ERROR, "Error message")
            log_router.log(libdnf5.logger.Logger.Level_WARNING,
                           "Warning message")
            log_router.log(libdnf5.logger.Logger.Level_NOTICE,
                           "Notice message")
            log_router.log(libdnf5.logger.Logger.Level_INFO, "Info message")
            log_router.log(libdnf5.logger.Logger.Level_DEBUG, "Debug message")
            log_router.log(libdnf5.logger.Logger.Level_TRACE, "Trace message")

        # ====================
        # 3. Create two StreamLogger instances which replace (swap) MemoryBufferLogger instance in log_router.
        # ====================
        # Create two streams for logging. Save pointers to them. Pointers will be needed later
        # in test of streams content.
        stream1 = io.StringIO()
        stream2 = io.StringIO()

        # Create StreamLogger instance and swap it with MemoryBufferLogger instance which was added
        # into LogRouter before.
        tmp_logger = libdnf5LoggerCB2(stream1)
        tmp_logger_uniq_ptr = libdnf5.logger.LoggerUniquePtr(tmp_logger)
        # In the log_router is registered only instance of MemoryBufferLogger just now.
        # The index of the first logger is "0".
        log_router.swap_logger(tmp_logger_uniq_ptr, 0)

        # Create secondary StreamLogger instance and add it to LogRouter as another logger.
        log2_stream = libdnf5LoggerCB2(stream2)
        log2_stream_uniq_ptr = libdnf5.logger.LoggerUniquePtr(log2_stream)
        log_router.add_logger(log2_stream_uniq_ptr)

        # Test the number of registered loggers.
        self.assertEqual(log_router.get_loggers_count(), 2)

        # ====================
        # 4. Write messages from original (replaced) MemoryBufferLoger instance into LogRouter instance.
        #    They will be routed to both attached stream loggers.
        # ====================
        memory_buffer_logger.write_to_logger(log_router)

        # ====================
        # 5. Write additional message into LogRouter instance.
        # ====================
        log_router.log(libdnf5.logger.Logger.Level_INFO,
                       "Info additional message")

        # ====================
        # 6. Check content of streams of both StreamLogger instances.
        # ====================
        content = stream1.getvalue()
        self.assertEqual(content, expected_logger_content)
        content = stream2.getvalue()
        self.assertEqual(content, expected_logger_content)
